# -*- coding: utf-8 -*-
import unittest


class TestVHM(unittest.TestCase):

    def setUp(self):
        import transaction
        from Testing.makerequest import makerequest
        from Testing.ZopeTestCase.ZopeLite import app
        transaction.begin()
        self.app = makerequest(app())
        if 'virtual_hosting' not in self.app.objectIds():
            # If ZopeLite was imported, we have no default virtual
            # host monster
            from Products.SiteAccess.VirtualHostMonster \
                import manage_addVirtualHostMonster
            manage_addVirtualHostMonster(self.app, 'virtual_hosting')
        self.app.manage_addFolder('folder')
        self.app.folder.manage_addDTMLMethod('doc', '')
        self.app.REQUEST.set('PARENTS', [self.app])
        self.traverse = self.app.REQUEST.traverse

    def tearDown(self):
        import transaction
        transaction.abort()
        self.app._p_jar.close()

    def test_url_with_host_header(self):
        req = self.app.REQUEST
        req.environ['HTTP_HOST'] = 'www.foobar.com'
        self.traverse('/folder/doc')
        self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                         'http://www.foobar.com/folder/doc')
        self.assertEqual(self.app.folder.doc.absolute_url(),
                         'http://www.foobar.com/folder/doc')

    def test_url_with_scheme_header(self):
        req = self.app.REQUEST
        req.environ['HTTP_X_SCHEME'] = 'https'
        self.traverse('/folder/doc')
        self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                         'https://foo/folder/doc')
        self.assertEqual(self.app.folder.doc.absolute_url(),
                         'https://foo/folder/doc')

    def test_url_with_scheme_and_host_header(self):
        req = self.app.REQUEST
        req.environ['HTTP_X_SCHEME'] = 'https'
        req.environ['HTTP_HOST'] = 'www.foobar.com'
        self.traverse('/folder/doc')
        self.assertEqual(self.app.REQUEST['ACTUAL_URL'],
                         'https://www.foobar.com/folder/doc')
        self.assertEqual(self.app.folder.doc.absolute_url(),
                         'https://www.foobar.com/folder/doc')



from castle.cms.interfaces import ISecureLoginAllowedView
from plone import api
from plone.registry.interfaces import IRegistry
from zExceptions import Redirect
from zope.component import queryUtility
from castle.cms import constants


SHIELD = constants.SHIELD

_blacklisted_meta_types = (
    'Image', 'File', 'Filesystem Image',
    'Filesystem File', 'Stylesheets Registry', 'JavaScripts Registry',
    'DirectoryViewSurrogate', 'KSS Registry', 'Filesystem Directory View')


def protect(req):
    published = req.PARENTS[0]
    mt = getattr(
        getattr(published, 'aq_base', None),
        'meta_type',
        getattr(published, 'meta_type', None))
    if mt in _blacklisted_meta_types or mt is None:
        return

    published = req.get('PUBLISHED')
    if ISecureLoginAllowedView.providedBy(published):
        return

    registry = queryUtility(IRegistry)
    setting = (registry and
               registry.get('plone.login_shield_setting', SHIELD.NONE) or
               SHIELD.NONE)

    protect = False
    if setting == SHIELD.ALL:
        protect = True
    elif setting == SHIELD.BACKEND:
        backend_url = (registry and
                       registry.get('plone.backend_url', SHIELD.NONE) or
                       '')
        if backend_url.startswith(req.SERVER_URL):
            protect = True
    if protect and api.user.is_anonymous():
        raise Redirect('{}/@@secure-login'.format(
            api.portal.get().absolute_url()))



import json

from castle.cms.tiles.base import BaseTile
from plone.memoize.instance import memoize
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from zope import schema
from zope.component import getUtility
from zope.schema.vocabulary import SimpleVocabulary


class TwitterTimelineTile(BaseTile):
    def render(self):
        screenName = self.data.get('screenName', '')

        widgetID = self.get_widget_id()

        if widgetID is None or screenName is None:
            return ''

        parameters = {}
        validFields = ITwitterTimelineTileSchema._InterfaceClass__attrs.keys()

        # Prevent any additional values from being passed to Twitter
        for key in self.data.keys():
            if key in validFields:
                parameters[key] = self.data.get(key, '')

        if 'screenName' in parameters:
            if parameters['screenName'][0] == '@':
                parameters['screenName'] = parameters['screenName'][1:]

        options = {
            'widgetId': widgetID,
            'parameters': parameters
        }

        self.patternOptions = json.dumps(options)
        self.portalUrl = self.context.portal_url()

        return self.index()

    @memoize
    def get_widget_id(self):
        registry = getUtility(IRegistry)
        return registry.get('plone.twitter_timeline_widget')


class ITwitterTimelineTileSchema(model.Schema):
    screenName = schema.TextLine(
        title=u"Screen Name",
        description=u"Screen name of the user whose timeline you want to embed."
    )

    height = schema.Int(
        title=u"Height",
        description=u"Height of the rendered timeline, in pixels.",
        default=400,
        min=200,
        required=False
    )

    tweetLimit = schema.Int(
        title=u"Tweet limit",
        description=u"Maximum # of tweets to show (1-20)",
        min=1,
        default=10,
        max=20,
        required=False
    )

    theme = schema.Choice(
        title=u"Theme",
        description=u"Timeline color scheme.",
        vocabulary=SimpleVocabulary.fromValues(['light', 'dark']),
        default='light',
        required=False
    )



# -*- coding: utf-8 -*-
from zope.annotation.interfaces import IAnnotations
from castle.cms import utils
from persistent.dict import PersistentDict
from persistent.list import PersistentList
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone.tiles.data import ANNOTATIONS_KEY_PREFIX as TILE_ANNOTATIONS_KEY_PREFIX

import unittest
import json
from castle.cms.browser import tiles as tile_views


class TestSlots(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING
    slot_id = 'meta-left-slot'

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))
        self._create_slot()

    def test_foobar(self):
        pass

    def _create_slot(self):
        annotations = IAnnotations(self.portal)
        tiles = PersistentList()
        for _ in range(2):
            tile_id = utils.get_random_string(30)
            annotations[TILE_ANNOTATIONS_KEY_PREFIX + '.' + tile_id] = PersistentDict({
                'foo': 'bar'
            })
            tiles.append({
                'id': tile_id
            })
        annotations[TILE_ANNOTATIONS_KEY_PREFIX + '.' + self.slot_id] = PersistentDict({
            'tiles': tiles
        })

    def ztest_copy_slot(self):
        view = tile_views.MetaTileManager(self.portal, self.request)
        self.request.form.update({
            'metaId': self.slot_id
        })
        new_id = json.loads(view.copy_meta())['newId']
        annotations = IAnnotations(self.portal)
        self.assertTrue(TILE_ANNOTATIONS_KEY_PREFIX + '.' + new_id in annotations)
        data = annotations[TILE_ANNOTATIONS_KEY_PREFIX + '.' + self.slot_id]
        self.assertEquals(data['locked']['user'], TEST_USER_ID)

    def ztest_copy_tiles(self):
        view = tile_views.MetaTileManager(self.portal, self.request)
        self.request.form.update({
            'metaId': self.slot_id
        })
        annotations = IAnnotations(self.portal)
        self.assertEqual(
            len([k for k in annotations.keys()
                 if k.startswith(TILE_ANNOTATIONS_KEY_PREFIX)]),
            3
        )
        view.copy_meta()
        self.assertEqual(
            len([k for k in annotations.keys()
                 if k.startswith(TILE_ANNOTATIONS_KEY_PREFIX)]),
            6
        )

    def ztest_save_slot(self):
        view = tile_views.MetaTileManager(self.portal, self.request)
        self.request.form.update({
            'metaId': self.slot_id
        })
        annotations = IAnnotations(self.portal)
        view.copy_meta()
        self.assertEqual(
            len([k for k in annotations.keys()
                 if k.startswith(TILE_ANNOTATIONS_KEY_PREFIX)]),
            6
        )
        view.save_copy()
        self.assertEqual(
            len([k for k in annotations.keys()
                 if k.startswith(TILE_ANNOTATIONS_KEY_PREFIX)]),
            3
        )

    def ztest_data_is_updated(self):
        view = tile_views.MetaTileManager(self.portal, self.request)
        self.request.form.update({
            'metaId': self.slot_id
        })
        annotations = IAnnotations(self.portal)
        view.copy_meta()
        version_key = view.get_working_copy_key()
        copy_slot = annotations[
            TILE_ANNOTATIONS_KEY_PREFIX + '.' + self.slot_id + '-' + version_key]
        tile = copy_slot['tiles'][0]
        tile_id = tile['id']
        tile_data = annotations[TILE_ANNOTATIONS_KEY_PREFIX + '.' + tile_id]
        tile_data['foo'] = 'bozo'
        view.save_copy()
        tile_data = annotations[
            TILE_ANNOTATIONS_KEY_PREFIX + '.' + tile_id.replace('-' + version_key, '')]

        self.assertEquals(tile_data['foo'], 'bozo')

    def ztest_tile_is_deleted(self):
        view = tile_views.MetaTileManager(self.portal, self.request)
        self.request.form.update({
            'metaId': self.slot_id
        })
        annotations = IAnnotations(self.portal)
        view.copy_meta()
        version_key = view.get_working_copy_key()
        copy_slot = annotations[
            TILE_ANNOTATIONS_KEY_PREFIX + '.' + self.slot_id + '-' + version_key]
        tile = copy_slot['tiles'][0]
        copy_slot['tiles'].remove(tile)
        tile_id = tile['id']
        del annotations[TILE_ANNOTATIONS_KEY_PREFIX + '.' + tile_id]
        view.save_copy()

        self.assertEqual(
            len([k for k in annotations.keys()
                 if k.startswith(TILE_ANNOTATIONS_KEY_PREFIX)]),
            2
        )

    def ztest_cancel_slot_edit(self):
        view = tile_views.MetaTileManager(self.portal, self.request)
        self.request.form.update({
            'metaId': self.slot_id
        })
        annotations = IAnnotations(self.portal)
        view.copy_meta()
        view.cancel_copy()

        self.assertEqual(
            len([k for k in annotations.keys()
                 if k.startswith(TILE_ANNOTATIONS_KEY_PREFIX)]),
            3
        )



# -*- coding: utf-8 -*-
from castle.cms import theming
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone.registry import field as registry_field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import unittest


MINIMAL_LAYOUT = """<!doctype html>
<html class="no-js" lang="en-us" data-gridsystem="bs3">
  <head>
    <meta charset="utf-8" />

    <link data-tile="${portal_url}/@@castle.cms.metadata"/>

    <link data-tile="${portal_url}/@@plone.app.standardtiles.stylesheets"/>

    <meta name="generator" content="Castle - https://www.wildcardcorp.com"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  </head>
  <body id="visual-portal-wrapper">
    <div data-tile="${context_url}/@@plone.app.standardtiles.toolbar" />

      <!-- Main Content -->
      <div class="row" id="main-content-container">
        <div id="main-content">
            ${structure: content.main}
        </div>
      </div>

      <link data-tile="${portal_url}/@@plone.app.standardtiles.javascripts"/>
  </body>
</html>
"""


MINIMAL_RESULT = '''<!doctype html>
<html>
  <head>
  </head>
  <body id="visual-portal-wrapper">
    <div>foobar</div>
  </body>
</html>'''


class TestTheming(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_get_layout(self):
        transform = theming.getTransform(self.portal, self.request)
        layout = transform.get_layout(self.portal)
        self.assertEqual(layout.name, 'index.html')

    def test_get_layout_from_environ(self):
        transform = theming.getTransform(self.portal, self.request)
        self.request.environ['X-CASTLE-LAYOUT'] = MINIMAL_LAYOUT
        layout = transform.get_layout(self.portal, request=self.request)
        self.assertEqual(layout.name, 'environ')

    def test_apply(self):
        transform = theming.getTransform(self.portal, self.request)
        result = transform(self.request, MINIMAL_RESULT, context=self.portal)
        self.assertTrue('<div>foobar</div>' in ''.join(result))
        self.assertTrue('data-pat-structure' in ''.join(result))

    def test_use_default_layout_in_registry(self):
        transform = theming.getTransform(self.portal, self.request)
        layout_name = transform.get_layout_name(self.portal)
        self.assertEqual(layout_name, 'index.html')
        registry = getUtility(IRegistry)
        field = registry_field.TextLine(title=u'Default layout', required=False)
        new_record = Record(field)
        registry.records['castle.cms.default_layout'] = new_record
        registry['castle.cms.default_layout'] = u'foobar.html'
        layout_name = transform.get_layout_name(self.portal)
        self.assertEqual(layout_name, 'foobar.html')



from plone import api
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import json
import requests


class PurgeManager(object):
    def __init__(self):
        registry = getUtility(IRegistry)
        self.api_key = registry.get('castle.cf_api_key', None)
        self.email = registry.get('castle.cf_email', None)
        self.zone_id = registry.get('castle.cf_zone_id', None)
        self.enabled = (
            self.api_key is not None and
            self.email is not None and
            self.zone_id is not None)
        self.site = api.portal.get()
        self.public_url = registry.get('plone.public_url', None)
        if not self.public_url:
            self.public_url = self.site.absolute_url()
        self.site_path = '/' + self.site.virtual_url_path()

    # eventual support more than just CF
    def getUrlsToPurge(self, path):

        # remove virtual hosting stuff
        if '/_vh_' in path:
            path = '/' + path.split('/_vh_')[-1].split('/', 1)[-1]
        if 'VirtualHostRoot' in path:
            path = path.split('VirtualHostRoot')[-1]
        if self.site_path not in ('', '/') and path.startswith(self.site_path):
            path = path[len(self.site_path):]

        urls = []
        urls.append('%s/%s' % (self.public_url.rstrip('/'), path.lstrip('/')))
        return urls

    def purge(self, urls):
        headers = {
            'X-Auth-Email': self.email,
            "X-Auth-Key": self.api_key,
            'Content-Type': 'application/json'
        }
        return requests.delete(
            'https://api.cloudflare.com/client/v4/zones/%s/purge_cache' % self.zone_id,
            headers=headers, data=json.dumps({'files': urls}))

def get():
    return PurgeManager()



import os
import argparse
import sys
import subprocess

parser = argparse.ArgumentParser(description='Run a script')
parser.add_argument('--instance', dest='instance',
                    help='path to instance executable. If not provided, '
                         'will look in bin this was executed from for '
                         'instance or client1')
parser.add_argument('--site-id', dest='siteid',
                    help='Some scripts care about site id')

this_dir = os.path.dirname(os.path.realpath(__file__))


def script_runner(script, argv=sys.argv):
    args, _ = parser.parse_known_args()
    instance = args.instance
    if not instance:
        # look for it, get bin directory, search for plone instance
        bin_path = os.path.sep.join(
            os.path.abspath(sys.argv[0]).split(os.path.sep)[:-1])
        files = os.listdir(bin_path)
        if 'instance' in files:
            instance = os.path.join(bin_path, 'instance')
        elif 'client1' in files:
            instance = os.path.join(bin_path, 'client1')
    if not instance:
        print("Could not find plone instance to run command against.")
        sys.exit()
    if script[0] != '/':
        script_path = os.path.join(this_dir, script)
    else:
        script_path = script
    cmd = [instance, 'run', script_path]

    print('Running command: %s' % ' '.join(cmd))
    subprocess.check_call(cmd, env=os.environ)


def run_it(module):
    from Zope2 import configure
    configure('/opt/plone/parts/client1/etc/zope.conf')
    import Zope2
    app = Zope2.app()
    from Testing.ZopeTestCase.utils import makerequest
    app = makerequest(app)
    app.REQUEST['PARENTS'] = [app]
    from zope.globalrequest import setRequest
    setRequest(app.REQUEST)
    from AccessControl.SpecialUsers import system as user
    from AccessControl.SecurityManagement import newSecurityManager
    newSecurityManager(None, user)

    from zope.dottedname.resolve import resolve
    mod = resolve('castle.cms.cron.' + module)
    mod.run(app)


def social_counts(argv=sys.argv):
    return run_it('_social_counts')


def clean_users(argv=sys.argv):
    return run_it('_clean_users')


def archive_content(argv=sys.argv):
    return run_it('_archive_content')


def ga_popularity(argv=sys.argv):
    return run_it('_popularity')


def empty_trash(argv=sys.argv):
    return run_it('_empty_trash')


def twitter_monitor(argv=sys.argv):
    return run_it('_twitter_monitor')


def reindex_es(argv=sys.argv):
    return run_it('_reindex_es')


def forced_publish_alert(argv=sys.argv):
    return run_it('_forced_publish_alert')


def crawler(argv=sys.argv):
    return run_it('_crawler')


def clean_drafts(argv=sys.argv):
    return run_it('_clean_drafts')



from castle.cms.browser.files import NamedFileDownload
from castle.cms.utils import site_has_icon
from cStringIO import StringIO
from PIL import Image
from plone import api
from plone.formwidget.namedfile.converter import b64decode_file
from plone.namedfile.file import NamedImage
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


class IconView(NamedFileDownload):
    """
    a bit insane all the icon sizes we need but this is the world we live in...


<link rel="apple-touch-icon" sizes="180x180" href="/site-icon.png">
<link rel="icon" type="image/png" href="/site-icon.png?scale=32" sizes="32x32">
<link rel="icon" type="image/png" href="//site-icon.png?scale=16" sizes="16x16">
<link rel="manifest" href="/manifest.json">

<!-- can't do this one!!! no way to convert to svg -->
<link rel="mask-icon" href="/site-icon.svg" color="#5bbad5">

<meta name="theme-color" content="#ffffff">


from PIL import Image
filename = r'logo.png'
img = Image.open(filename)
icon_sizes = [(16,16), (32, 32), (48, 48), (64,64)]
img.save('logo.ico', sizes=icon_sizes)

    """

    filename = 'icon.png'

    allowed_sizes = (
        16,
        32,
        150,
        180,
        192,
        512
    )

    def get_data(self):
        registry = getUtility(IRegistry)
        try:
            data = registry['plone.site_icon']
        except:
            data = None
        if data:
            filename, data = b64decode_file(data)
            return data

    def scale_data(self, data, scale):
        result = StringIO()
        image = Image.open(StringIO(data))
        image.thumbnail((scale, scale))
        image.save(result, "PNG")
        result.seek(0)
        return result.read()

    def _getFile(self):
        scale = None
        if 'scale' in self.request.form:
            try:
                scale = int(self.request.form['scale'])
            except:
                pass
        data = self.get_data()
        if data:
            if scale is not None:
                data = self.scale_data(data, scale)
            return NamedImage(data=data, filename=self.filename)


class FaviconView(IconView):
    filename = 'favicon.ico'

    def scale_data(self, data):
        result = StringIO()
        image = Image.open(StringIO(data))
        image.save(result, "ICO")
        result.seek(0)
        return result.read()

    def _getFile(self):
        data = self.get_data()
        if data:
            data = self.scale_data(data)
            return NamedImage(data=data, filename=self.filename)


class ManifestView(NamedFileDownload):

    def __call__(self):
        self.request.response.setHeader('Content-type', 'application/json')
        title = api.portal.get_registry_record('plone.short_site_title')
        if not title:
            title = api.portal.get_registry_record('plone.site_title')
        if not site_has_icon():
            return '{"name": "%(site_title)s"}' % dict(site_title=title)

        return '''{
    "name": "%(site_title)s",
    "icons": [
        {
            "src": "%(url)s/site-icon.png?scale=192",
            "sizes": "192x192",
            "type": "image\/png"
        },
        {
            "src": "%(url)s/site-icon.png?scale=512",
            "sizes": "512x512",
            "type": "image\/png"
        }
    ],
    "theme_color": "#ffffff",
    "display": "standalone"
}''' % dict(url=self.context.absolute_url(),
            site_title=title)


class BrowserConfigView(NamedFileDownload):

    def __call__(self):
        self.request.response.setHeader('Content-type', 'text/xml')
        if not site_has_icon():
            return '''<?xml version="1.0" encoding="utf-8"?>
    <browserconfig>
    </browserconfig>'''

        return '''<?xml version="1.0" encoding="utf-8"?>
<browserconfig>
  <msapplication>
    <tile>
      <square150x150logo src="{url}/site-icon.png?scale=150"/>
      <TileColor>#da532c</TileColor>
    </tile>
  </msapplication>
</browserconfig>
'''.format(url=self.context.absolute_url())



from castle.cms.tiles.base import BaseTile
from plone.supermodel import model
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary

import json
import re


class TweetTile(BaseTile):
    def render(self):
        content = self.data.get('url', '') or ''

        urlRegEx = ".*twitter\.com/.*/status/([0-9]+)"
        urlParser = re.compile(urlRegEx)
        matches = urlParser.match(content)

        if matches is None:
            return ''

        tweetID = matches.group(1)
        options = {'tweetID': tweetID}

        options['settings'] = {}

        cards = self.data.get('cards', '')
        options['settings']['cards'] = cards

        self.patternOptions = json.dumps(options)
        self.portalUrl = self.context.portal_url()

        return self.index()


class ITweetTileSchema(model.Schema):
    url = schema.URI(
        title=u"URL",
        description=u"URL to the tweet to include in the tile."
    )

    cards = schema.Choice(
        title=u"Hide tweet card",
        default='visible',
        description=u"Whether or not to include any associate pictures or videos.",
        vocabulary=SimpleVocabulary.fromValues(['visible','hidden'])
    )



# -*- coding: utf-8 -*-
from castle.cms.browser import content
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.uuid.interfaces import IUUID

import json
import unittest


class TestContent(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_dump_data(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        data = content.dump_object_data(doc)
        self.assertEqual(json.loads(data)['uid'], IUUID(doc))

    def test_dump_data_with_view(self):
        folder = api.content.create(
            type='Folder',
            id='file-repository',
            container=self.portal)
        doc = api.content.create(
            type='File',
            id='fi',
            container=folder)
        data = content.dump_object_data(doc)
        self.assertEqual(json.loads(data)['url'], doc.absolute_url() + '/view')

    def test_workflow_transition(self):
        doc = api.content.create(
            type='Document',
            id='doc',
            container=self.portal)
        self.request.form.update({
            'transition_id': 'publish',
            'action': 'transition'
        })
        wf = content.Workflow(doc, self.request)
        wf()
        self.assertEqual(api.content.get_state(doc), 'published')

    def test_check(self):
        self.request.form.update({
            'action': 'check',
            'selectedType[typeId]': 'Folder',
            'basePath': '/',
            'id': 'foobar'
        })
        cc = content.Creator(self.portal, self.request)
        data = cc()
        self.assertEqual(json.loads(data)['valid'], True)


class TestPageLayoutSelector(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_get_front_page(self):
        view = content.PageLayoutSelector(self.portal['front-page'], self.request)
        result = json.loads(view())
        self.assertEqual(result['success'], True)
        self.assertEqual(result['page_layout'], 'frontpage.html')
        self.assertEqual(result['section_layout'], None)

    def test_set_front_page(self):
        view = content.PageLayoutSelector(self.portal['front-page'], self.request)
        self.request.form.update({
            'data': json.dumps({
                'page_layout': 'foobar.html',
                'section_layout': 'foobar.html'
            }),
            'action': 'save'
        })
        view()

        # check results...
        del self.request.form['action']
        result = json.loads(view())
        self.assertEqual(result['success'], True)
        self.assertEqual(result['page_layout'], 'foobar.html')
        self.assertEqual(result['section_layout'], 'foobar.html')

        # also check that the registry entry value set since front page is default
        # page for site
        self.assertEqual(
            self.portal.portal_registry['castle.cms.default_layout'],
            'foobar.html')



from Products.CMFPlone.browser import contact_info
from Products.CMFPlone.browser.interfaces import IContactForm as IBaseContactForm
from Products.statusmessages.interfaces import IStatusMessage
from castle.cms.utils import verify_recaptcha
from castle.cms.widgets import ReCaptchaFieldWidget
from plone import api
from plone.registry.interfaces import IRegistry
from z3c.form import button
from z3c.form import field
from z3c.form.action import ActionErrorOccurred
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import WidgetActionExecutionError
from zope import schema
from zope.component import getUtility
from zope.event import notify
from zope.interface import Invalid


class IContactForm(IBaseContactForm):
    captcha = schema.TextLine(
        title=u"Captcha",
        required=False)


class ContactForm(contact_info.ContactForm):

    fields = field.Fields(IContactForm)

    def update(self):
        super(ContactForm, self).update()
        registry = getUtility(IRegistry)
        has_captcha = registry.get('castle.recaptcha_private_key') is not None
        if not has_captcha:
            self.fields['captcha'].mode = HIDDEN_MODE
        else:
            self.fields['captcha'].widgetFactory = ReCaptchaFieldWidget

    @button.buttonAndHandler(u'Send', name='send')
    def handle_send(self, action):
        data, errors = self.extractData()

        registry = getUtility(IRegistry)
        has_captcha = registry.get('castle.recaptcha_private_key') is not None

        if has_captcha:
            if not verify_recaptcha(self.request):
                notify(
                    ActionErrorOccurred(
                        action,
                        WidgetActionExecutionError('captcha', Invalid('Invalid Recaptcha'))))
                return

        if errors:
            IStatusMessage(self.request).add(
                self.formErrorsMessage,
                type=u'error'
            )

            return

        self.send_message(data)
        self.send_feedback()
        self.success = True

    def render(self):
        registry = getUtility(IRegistry)
        if registry.get('plone.disable_contact_form', False):
            self.request.response.redirect(api.portal.get().absolute_url())
        return super(ContactForm, self).render()


from plone.tiles.interfaces import IPersistentTile
from zope.interface import Interface


class IGlobalTile(IPersistentTile):
    pass


class IMetaTile(IPersistentTile):
    pass


class IFieldTileRenderer(Interface):
    pass



from apiclient.discovery import build
from oauth2client.client import SignedJwtAssertionCredentials
from plone.formwidget.namedfile.converter import b64decode_file
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import httplib2


def get_service(api_name, api_version, scope, key, service_account_email):
    """Get a service that communicates to a Google API.

    Args:
        api_name: The name of the api to connect to.
        api_version: The api version to connect to.
        scope: A list auth scopes to authorize for the application.
        key_file_location: The path to a valid service account p12 key file.
        service_account_email: The service account email address.

    Returns:
        A service that is connected to the specified API.
    """

    credentials = SignedJwtAssertionCredentials(
        service_account_email, key, scope=scope)

    http = credentials.authorize(httplib2.Http())

    # Build the service object.
    service = build(api_name, api_version, http=http)

    return service


def get_ga_service(ga_scope=['https://www.googleapis.com/auth/analytics.readonly']):
    registry = getUtility(IRegistry)
    api_email = registry.get('castle.google_api_email', None)
    api_key = registry.get('castle.google_api_service_key_file', None)
    ga_id = registry.get('castle.google_analytics_id', None)

    if not api_key or not api_email or not ga_id:
        return

    # Authenticate and construct service.
    return get_service(
        'analytics', 'v3', ga_scope, b64decode_file(api_key)[1], api_email)


def get_ga_profile(service):
    registry = getUtility(IRegistry)
    ga_id = registry.get('castle.google_analytics_id', None)
    if not ga_id:
        return

    # Use the Analytics service object to get the first profile id.

    # Get a list of all Google Analytics accounts for this user
    accounts = service.management().accounts().list().execute()

    if accounts.get('items'):
        # Get the first Google Analytics account.
        account = accounts.get('items')[0].get('id')

        # Get a list of all the properties for the first account.
        properties = service.management().webproperties().list(
            accountId=account).execute()

        for item in properties['items']:
            if ga_id == item['id']:
                # Get a list of all views (profiles) for the first property.
                profiles = service.management().profiles().list(
                    accountId=account,
                    webPropertyId=ga_id).execute()

                if profiles.get('items'):
                    # return the first view (profile) id.
                    return profiles.get('items')[0].get('id')

    return None



# -*- coding: utf-8 -*-
from castle.cms import indexers
from castle.cms.interfaces import IReferenceNamedImage
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms.widgets import LeadImageFocalNamedImageFieldWidget
from plone import api
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.namedfile.file import NamedBlobImage
from plone.namedfile.tests.test_image import zptlogo
from plone.uuid.interfaces import IUUID
from z3c.form.interfaces import IDataConverter
from zope.component import getMultiAdapter

import unittest


b64_image = "data:image/gif;base64,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"  # noqa


class BaseTest(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def _get_image(self):
        return NamedBlobImage(zptlogo, contentType='image/jpeg')

    def _get_widget(self, context=None):
        if context is None:
            context = api.content.create(type='Document', title='foobar',
                                         container=self.portal)
        widget = LeadImageFocalNamedImageFieldWidget(
            ILeadImage['image'],
            self.request
        )
        widget.context = context
        widget.id = 'widget.id.image'
        widget.name = 'widget.name.image'
        widget.value = self._get_image()

        return widget

    def _get_converter(self, widget=None, context=None):
        if widget is None:
            widget = self._get_widget(context=context)
        return getMultiAdapter((ILeadImage['image'], widget), IDataConverter)

    def _get_value(self, context):
        converter = self._get_converter(context=context)
        self.request.form.update({
            converter.widget.name + '.filename': 'image.gif',
            converter.widget.name + '.focalX': '1',
            converter.widget.name + '.focalY': '1'
        })
        return converter.toFieldValue(b64_image)

    def _get_object(self, reference=False):
        context = api.content.create(type='Document', title='foobar2',
                                     container=self.portal)
        if reference:
            widget = self._get_widget()
            converter = self._get_converter(widget)
            context.image = converter.toFieldValue('reference:' + IUUID(context))
        else:
            context.image = self._get_value(context)
        return context


class TestImageWidget(BaseTest):

    def test_save_image_focal_point(self):
        widget = self._get_widget()
        converter = self._get_converter(widget)
        self.request.form.update({
            widget.name + '.filename': 'image.gif',
            widget.name + '.focalX': '1',
            widget.name + '.focalY': '1'
        })
        result = converter.toFieldValue(b64_image)
        self.assertTrue(hasattr(result, 'focal_point'))
        self.assertTrue(hasattr(widget.context, '_image_focal_point'))

    def test_save_image(self):
        widget = self._get_widget()
        converter = self._get_converter(widget)
        self.request.form.update({
            widget.name + '.filename': 'image.gif'
        })
        result = converter.toFieldValue(b64_image)
        self.assertEquals(type(result), NamedBlobImage)

    def test_save_image_reference(self):
        context = api.content.create(type='Document', title='foobar2',
                                     container=self.portal)
        widget = self._get_widget()
        converter = self._get_converter(widget)
        result = converter.toFieldValue('reference:' + IUUID(context))
        self.assertTrue(IReferenceNamedImage.providedBy(result))


class TestIndexImage(BaseTest):

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_index_focal_point(self):
        context = self._get_object()
        value = indexers.image_info(context)()
        self.assertEquals(value['focal_point'], [1, 1])

    def test_index_reference(self):
        reference = api.content.create(type='Document', title='foobar2',
                                       container=self.portal)
        context = self._get_object(reference)
        indexer = indexers.image_info(context)
        value = indexer()
        self.assertEquals(value['reference'], IUUID(context))



# -*- coding: utf-8 -*-
from castle.cms.testing import BaseTest
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from plone.namedfile.file import NamedBlobImage
from plone.namedfile.tests.test_image import zptlogo
from Products.CMFPlone.browser.syndication.settings import FeedSettings

import json


class TestSyndication(BaseTest):

    def _get_image(self):
        return NamedBlobImage(zptlogo, contentType='image/jpeg',
                              filename=u'foobar.jpg')

    def test_json_feed_404_when_not_active(self):
        settings = FeedSettings(self.portal)
        settings.feed_types = []
        resp = self.publish('/plone/feed.json')
        self.assertEqual(resp.status_code, 404)

    def test_json_feed_when_active(self):
        settings = FeedSettings(self.portal)
        settings.feed_types = ['feed.json']
        resp = self.publish('/plone/feed.json')
        self.assertEqual(resp.status_code, 200)

    def test_json_feed_contents(self):
        settings = FeedSettings(self.portal)
        settings.feed_types = ['feed.json']
        resp = self.publish('/plone/feed.json')
        result = json.loads(resp.body)
        self.assertEqual(result['title'], u'Plone site')
        self.assertEqual(
            len(result['items']), len(self.portal.portal_catalog()))

    def test_image_url_with_json(self):
        settings = FeedSettings(self.portal)
        settings.feed_types = ['feed.json']
        page = self.portal['front-page']
        page.image = self._get_image()

        resp = self.publish('/plone/feed.json')
        result = json.loads(resp.body)
        base_url = page.absolute_url()
        self.assertEqual(
            result['items'][0]['image_url'],
            '{}/@@images/image'.format(base_url))
        self.assertEqual(
            result['items'][0]['file_url'],
            '{}/@@download/image/{}'.format(base_url, page.image.filename))



from castle.cms import install
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from Products.CMFPlone.browser.admin import AddPloneSite
from Products.CMFPlone.browser.admin import Overview
from Products.CMFPlone.factory import _DEFAULT_PROFILE
from Products.CMFPlone.factory import addPloneSite
from Products.CMFPlone.resources.browser.combine import combine_bundles
from zope.interface import alsoProvides

try:
    from sites import get_site_container
except ImportError:
    get_site_container = False

import json
import logging
import transaction


LOGGER = logging.getLogger('Products.CMFPlone')


DYNAMIC_STORAGE_ENABLED = False
if get_site_container is None:
    # requires zope version with this
    DYNAMIC_STORAGE_ENABLED = False


class AddCastleSite(AddPloneSite):

    def buildSite(self, form):
        # CSRF protect. DO NOT use auto CSRF protection for adding a site
        alsoProvides(self.request, IDisableCSRFProtection)
        self.request.response.setHeader('Content-type', 'application/json')

        extension_ids = form.get('extension_ids[]', [])
        extension_ids.append('castle.cms:default')

        # TODO: barceloneta needs to just be removed
        if 'plonetheme.barceloneta:default' in extension_ids:
            extension_ids.remove('plonetheme.barceloneta:default')

        site_id = form.get('site_id', 'Castle')

        try:
            site = addPloneSite(
                self.get_creation_context(site_id),
                site_id,
                title=form.get('title', ''),
                profile_id=form.get('profile_id', _DEFAULT_PROFILE),
                extension_ids=extension_ids,
                setup_content=form.get('setup_content', True),
                default_language=form.get('default_language', 'en'),
                portal_timezone=form.get('portal_timezone', 'UTC')
            )

            # delete news/events since they have default pages attached to them
            api.content.delete(objects=(site['news'], site['events']))
            install.tiles(site, self.request)

            # need to reindex
            catalog = api.portal.get_tool('portal_catalog')
            catalog.manage_catalogRebuild()

            combine_bundles(site)

            if DYNAMIC_STORAGE_ENABLED:
                url = '{}/sites/{}'.format(self.context.absolute_url(), site_id)
            else:
                url = '{}/{}'.format(self.context.absolute_url(), site_id)

            # we do not allow basic auth
            acl_users = site.acl_users
            acl_users.manage_delObjects(['credentials_basic_auth'])

            return json.dumps({
                'success': True,
                'url': url
            })

        except Exception as e:
            LOGGER.error('Error creating site', exc_info=True)
            transaction.abort()  # abort creating this site if there was an error
            return json.dumps({'success': False, 'msg': str(e)})

    def get_creation_context(self, site_id):
        if not DYNAMIC_STORAGE_ENABLED:
            return self.context

        site_app = get_site_container(site_id)
        site_app = site_app.__of__(self.context)
        return site_app

    def __call__(self):

        def labeled_map(labelName, valueName, values):
            return {x[labelName]: x[valueName] for x in values}

        form = self.request.form
        submitted = form.get('submitted', False)

        if submitted:
            return self.buildSite(form)

        timezone_data = self.timezones()
        timezones = {x: labeled_map('label', 'value', timezone_data[x]) for x in timezone_data}

        defaultLanguage = self.browser_language()
        language_data = self.grouped_languages(defaultLanguage)
        languages = {x['label']: labeled_map('label', 'langcode', x['languages'])
                     for x in language_data}

        form_data = {}
        form_data['default_language'] = defaultLanguage
        form_data['timezones'] = timezones
        form_data['languages'] = languages
        form_data['action_url'] = self.context.absolute_url() + '/@@plone-addsite'

        profiles = self.profiles()['extensions']
        addons = {}
        for profile in profiles:

            if 'selected' in profile:
                selected = True
            else:
                selected = False

            addons[profile['id']] = {
                'description': profile['description'],
                'title': profile['title'],
                'checked': selected,
                'type': 'checkbox'
            }
        form_data['extensions'] = addons
        self.form_data = json.dumps(form_data)
        return self.index()


class Overview(Overview):

    def __call__(self):
        return self.index()



from castle.cms.interfaces import ITrashed
from plone import api
from plone.app.standardtiles import common
from plone.app.standardtiles import head
from plone.locking.browser.info import LockInfoViewlet
from plone.locking.interfaces import ITTWLockable
from plone.tiles import Tile


class LockInfoTile(Tile):
    def __call__(self):
        if not ITTWLockable.providedBy(self.context):
            return

        viewlet = LockInfoViewlet(self.context, self.request, None, None)
        return viewlet.render()


class GlobalStatusMessageTile(common.GlobalStatusMessageTile):

    def __call__(self):
        # we're overriding so we can provide extra status messages
        # just an easy way to hook in
        if ITrashed.providedBy(self.context):
            # warn user they are in no-no land
            api.portal.show_message(
                'You are viewing content that is in the recycling bin. '
                'You are still free to view and manage this object '
                'before it is restored or deleted.',
                request=self.request, type='error')
        return super(GlobalStatusMessageTile, self).__call__()



from BTrees.OOBTree import OOBTree
from castle.cms import theming
from castle.cms.files import aws
from castle.cms.interfaces import IArchiveContentTransformer
from castle.cms.utils import normalize_url
from DateTime import DateTime
from lxml.html import fromstring
from lxml.html import tostring
from plone import api
from plone.subrequest import subrequest
from plone.uuid.interfaces import IUUID
from urlparse import urljoin
from urlparse import urlparse
from zope.component import getAllUtilitiesRegisteredFor
from zope.globalrequest import getRequest
from zope.interface import implements

import hashlib
import logging
import re
import requests


logger = logging.getLogger('castle.cms')


# gets all url() CSS directives
RE_CSS_URL = re.compile(r"""url\(["']?([^\)'"]+)['"]?\)""")
# inline css import...
RE_CSS_IMPORTS = re.compile(r"""\@import ["']([a-zA-Z0-9\+\.\-\/\:\_]+\.(?:css))["'];""")

CONTENT_KEY_PREFIX = 'archives/'
RESOURCES_KEY_PREFIX = 'archiveresources/'


class BaseArchivalTransformer(object):
    implements(IArchiveContentTransformer)
    def __init__(self, archiver):
        self.archiver = archiver

    def __call__(dom):
        pass


def getContentToArchive(delta=0):
    days = api.portal.get_registry_record(
        'castle.archival_number_of_days') - delta
    types = api.portal.get_registry_record(
        'castle.archival_types_to_archive')
    states = api.portal.get_registry_record(
        'castle.archival_states_to_archive')
    if not types or not states:
        return []
    catalog = api.portal.get_tool('portal_catalog')
    end = DateTime()
    if days > 0:
        end -= days  # we're looking for data last modified
    query = dict(
        portal_type=types,
        modified={
            'query': (DateTime('1999/09/09'), end),
            'range': 'min:max'
        },
        sort_on="modified",
        review_state=states)
    return catalog(**query)


class ImageResourceMover(object):
    keep_ext = False

    attr = 'src'
    selector = 'img'

    def __init__(self, dom):
        self.dom = dom

    def get_elements(self):
        return self.dom.cssselect(self.selector)

    def get_url(self, el):
        return el.attrib.get(self.attr)

    def modify(self, el, new_url):
        el.attrib['original-url'] = el.attrib[self.attr]
        el.attrib[self.attr] = new_url


class LinkResourceMover(ImageResourceMover):
    attr = 'href'
    selector = 'link[rel="stylesheet"]'


class StyleResourceMover(ImageResourceMover):
    selector = 'style[type="text/css"]'

    def get_url(self, el):
        if not el.text:
            return
        if not el.text.startswith('@import'):
            return None
        return el.text.strip().lstrip('@import url(').rstrip(');')

    def modify(self, el, new_url):
        el.text = el.text.replace(self.get_url(el), new_url)


class ScriptResourceMover(ImageResourceMover):
    attr = 'src'
    selector = 'script[src]'


class EmbedResourceMover(ImageResourceMover):
    attr = 'src'
    selector = 'embed[src]'


class ObjectResourceMover(ImageResourceMover):
    attr = 'data'
    selector = 'object[data]'


class FlashScriptResourceMover(ImageResourceMover):
    """
    Example:
    <script type="text/javascript">
AC_FL_RunContent('codebase','http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,28,0','width','920','height','670','src','foobar','quality','high','wmode','transparent','pluginspage','http://www.adobe.com/shockwave/download/download.cgi?P1_Prod_Version=ShockwaveFlash','movie','foobar' );
</script>
    """

    keep_ext = True

    def get_elements(self):
        results = []
        for script in self.dom.cssselect('script'):
            if 'src' in script.attrib:
                continue
            if 'AC_FL_RunContent' in script.text:
                results.append(script)
        return results

    def get_rel_url(self, el):
        script = None
        for line in el.text.strip().splitlines():
            line = line.strip()
            if line.startswith('//'):
                continue
            if ' //' in line:
                line = line.split(' //')[0]
            if 'AC_FL_RunContent' in line:
                script = line
                break
        if script:
            params = script.replace('AC_FL_RunContent(', '').replace(');', '').split(',')
            swf = params[-1].replace('"', '').replace("'", '').strip()
            return swf

    def get_url(self, el):
        swf = self.get_rel_url(el)

        if not swf:
            return

        swf = swf.split('?')[0]

        if not swf.lower().endswith('.swf'):
            swf += '.swf'

        # if relative use base tag...
        if swf[0] == '/' or 'https://' in swf or 'http://' in swf:
            return swf

        base = self.dom.cssselect('base[href]')
        if len(base) > 0:
            swf = urljoin(base[0].attrib['href'], swf)
        else:
            body = self.dom.cssselect('body')[0]
            if 'data-portal-url' in body.attrib:
                swf = urljoin(body.attrib['data-portal-url'], swf)
        return swf

    def modify(self, el, new_url):
        swf = self.get_rel_url(el)
        el.text = el.text.replace(swf, new_url.replace('.swf', ''))


def _get_vhm_base_url(public_url, site_path):
    if not public_url:
        return site_path
    parsed = urlparse(public_url)
    return '/VirtualHostBase/%s/%s:%i%s/VirtualHostRoot' % (
        parsed.scheme,
        parsed.netloc,
        parsed.scheme == 'http' and 80 or 443,
        site_path
    )


class RequestsUrlOpener(object):

    def __init__(self, migrator):
        self.migrator = migrator
        self.site = migrator.site

    def __call__(self, url):
        url = normalize_url(url)
        if not url:
            return

        resp = requests.get(url)
        if resp.status_code in (404, 403, 401, 500, 501, 502):
            return
        if len(resp.history) > 0:
            # if it was a redirect and came_from in the url
            if resp.history[-1].status_code in (301, 302) and 'came_from' in resp.url:
                return

        return {
            'data': resp.content,
            'headers': resp.headers,
            'code': resp.status_code
        }


class SubrequestUrlOpener(object):
    _blacklisted_content = (
        '/refresh.png',
        '/contenttypes-sprite.png',
        '/glyphicons-halflings-regular.ttf',
        '/glyphicons-halflings-regular.svg',
        '/%23gradient',
        '/pb_close.png',
        '/delete.png',
        '/undelete.png',
        '/event_icon.png',
        '/++resource++plone.app.jquerytools.next.gif',
        '/++resource++plone.app.jquerytools.prev.gif',
        '/++resource++plone.app.jquerytools.pb_close.png',
        '/popup_calendar.png',
        '/next.gif',
        '/prev.gif'
    )

    def __init__(self, migrator):
        self.migrator = migrator
        self.site = migrator.site
        self.site_path = '/'.join(self.site.getPhysicalPath())
        self.public_url = api.portal.get_registry_record('plone.public_url')
        if not self.public_url:
            self.public_url = self.site.absolute_url()
        self.vhm_base = _get_vhm_base_url(self.public_url, self.site_path)

    def __call__(self, url):
        url = normalize_url(url)
        if not url:
            return

        if not url.startswith(self.public_url):
            return

        if '++plone++production' in url:
            front, end = url.rsplit('/', 1)
            # check blacklist
            for black_listed in self._blacklisted_content:
                if url.startswith(front + black_listed):
                    return

        # since we're looking at plone here... let's try fixing up urls...
        if '++plone++' in url:
            # can always be from site root
            url = self.public_url + '/++plone++' + url.rsplit('++plone++', 1)[-1]

        parsed = urlparse(url)
        vhm_path = self.vhm_base + parsed.path
        resp = subrequest(vhm_path)
        if resp.getStatus() == 404:
            return

        return {
            'data': resp.getBody(),
            'headers': resp.headers,
            'code': resp.getStatus()
        }


class Storage(object):
    """
    We store data on content we've moved:
        1) new url in AWS
        2) UID of obj we moved -> for resolveuid issues
        3) path of obj -> for regular 404s
    """

    Movers = (
        ImageResourceMover,
        LinkResourceMover,
        StyleResourceMover,
        ScriptResourceMover,
        FlashScriptResourceMover,
        ObjectResourceMover,
        EmbedResourceMover
    )
    _s3_conn = _bucket = None

    def __init__(self, site, UrlOpener=SubrequestUrlOpener):
        self.site = site
        self.url_opener = UrlOpener(self)
        self.public_url = api.portal.get_registry_record('plone.public_url')
        try:
            self.archives = self.site._archives
            self.path_to_uid = self.site._archives_path_to_uid
        except AttributeError:
            self.archives = self.site._archives = OOBTree()
            self.path_to_uid = self.site._archives_path_to_uid = OOBTree()
        self.resources = {}  # path -> md5 dict
        self.errors = []  # so we can ignore

        try:
            self.replacements = api.portal.get_registry_record(
                'castle.archival_replacements') or {}
        except:
            self.replacements = {}

        self.view_url_types = api.portal.get_registry_record(
            'plone.types_use_view_action_in_listings')

    @property
    def s3_conn(self):
        if self._s3_conn is None:
            self._initialize_s3()
        return self._s3_conn

    @property
    def bucket(self):
        if self._bucket is None:
            self._initialize_s3()
        return self._bucket

    def _initialize_s3(self):
        bucket_name = api.portal.get_registry_record('castle.aws_s3_bucket_name')
        self._s3_conn, self._bucket = aws.get_bucket(bucket_name)

    def apply_replacements(self, content):
        # first pass is for straight text
        for key, val in self.replacements.items():
            if key[0] == '.' or key[1] == '#':
                continue
            content = content.replace(key, val)

        dom = fromstring(content)
        for key, val in self.replacements.items():
            if key[0] != '.' and key[1] != '#':
                continue
            el = dom.cssselect(key)
            if len(el) > 0:
                el[0].text = val
                if 'style' in el[0].attrib:
                    del el[0].attrib['style']
                # also check parent
                parent = el[0].getparent()
                if 'style' in parent.attrib:
                    del parent.attrib['style']

        return tostring(dom)

    def move_to_aws(self, content, content_path, content_type='text/html; charset=utf-8',
                    replace=True):
        # perform replacements
        if 'html' in content_type and self.replacements:
            content = self.apply_replacements(content)
        content_path = content_path.lstrip('/')
        content_path = CONTENT_KEY_PREFIX + content_path
        url = 'https://{host}/{bucket}/{key}'.format(
            host=self.s3_conn.server_name(),
            bucket=self.bucket.name,
            key=content_path)
        key = self.bucket.new_key(content_path)
        key.set_contents_from_string(content, headers={
            'Content-Type': content_type
        }, replace=True)
        key.make_public()
        return url

    def move_resource(self, url, keep_ext=False):
        if 'data:' in url:
            return
        if url in self.errors:
            print('skipping because of error %s' % url)
            return
        resp = self.url_opener(url)
        if resp is None:
            self.errors.append(url)
            return
        logger.info('moving url: %s - %i' % (url, resp['code']))
        fidata = resp['data']

        # parse response, look for additional urls in content that need to be
        # moved over
        if 'text' in resp['headers'].get('content-type', '').lower():
            for sub_url in RE_CSS_URL.findall(fidata) + RE_CSS_IMPORTS.findall(fidata):
                resource_url = sub_url
                if not sub_url.startswith('http'):
                    resource_url = urljoin(url, sub_url)
                if resource_url not in self.resources:
                    moved_url = self.move_resource(resource_url)
                    if moved_url:
                        self.resources[resource_url] = moved_url
                if resource_url in self.resources:
                    new_url = self.resources[resource_url]
                    fidata = fidata.replace(sub_url, new_url)

        # upload to amazon and get url!
        md5 = hashlib.md5(fidata).hexdigest()

        content_path = '{0}{1}/{2}/{3}/{4}'.format(
            RESOURCES_KEY_PREFIX, md5[0], md5[1], md5[2], md5
        )
        if keep_ext and '.' in url:
            ext = url.split('.')[-1]
            content_path += '.' + ext
        new_url = 'https://{host}/{bucket}/{key}'.format(
            host=self.s3_conn.server_name(),
            bucket=self.bucket.name,
            key=content_path)

        # first check if already moved
        if self.bucket.get_key(content_path) is None:
            key = self.bucket.new_key(content_path)
            key.set_contents_from_string(fidata, headers={
                'Content-Type': resp['headers']['content-type'],
                'Content-Disposition': resp['headers'].get('content-disposition')
            }, replace=True)
            key.make_public()

        return new_url

    def add_url(self, url, content_path, uid):
        resp = self.url_opener(url)
        if resp is None:
            return
        content = resp['data']
        if 'html' in resp['headers']['content-type']:
            content = self.transform_content(content, url)
        new_url = self.move_to_aws(content, content_path,
                                   content_type=resp['headers']['content-type'])
        self.archives[uid] = {
            'path': content_path,
            'url': new_url
        }
        self.path_to_uid[content_path] = uid
        return new_url

    def transform_content(self, content, from_url):
        parsed_url = urlparse(from_url)
        domain = parsed_url.netloc
        dom = fromstring(content)
        for Mover in self.Movers:
            mover = Mover(dom)
            for el in mover.get_elements():
                url = mover.get_url(el)
                if url is None:
                    continue

                if url[0] == '/':
                    url = '{}://{}{}'.format(parsed_url.scheme, domain, url)
                elif 'https://' not in url and 'http://' not in url:
                    url = urljoin(from_url, url)

                # check that the url is on the site...
                rdomain = urlparse(url).netloc
                if rdomain and domain != rdomain:
                    continue
                if url not in self.resources:
                    # need to move resource
                    resource_url = url
                    if not url.startswith('http'):
                        resource_url = urljoin(from_url, url)
                    moved_url = self.move_resource(resource_url, mover.keep_ext)
                    if moved_url:
                        self.resources[url] = moved_url
                if url in self.resources:
                    mover.modify(el, aws.swap_url(self.resources[url]))
        content = tostring(dom)
        for Util in getAllUtilitiesRegisteredFor(IArchiveContentTransformer):
            try:
                util = Util(self)
                content = util(content)
            except:
                logger.info('Error with archive utility', exc_info=True)
        return content

    def massage_plone_resp_content(self, ob, resp, url):
        content = resp['data']
        if 'html' in resp['headers']['content-type']:
            if resp['headers'].get('x-theme-applied') != 'true':
                # annoying case where theme isn't applied here...

                req = getRequest()

                # XXX a bit weird, we need to virtualhostmonster it for transform...
                site_path = '/'.join(self.site.getPhysicalPath())
                public_url = api.portal.get_registry_record('plone.public_url')
                vhm_base = _get_vhm_base_url(public_url, site_path)
                req.traverse(vhm_base)

                transform = theming.getTransform(ob, req)
                content = ''.join(transform(req, content, context=ob))
            content = self.transform_content(content, url)
        return content

    def add_content(self, ob):
        site_path = '/'.join(self.site.getPhysicalPath())
        aws_content_path = content_path = '/'.join(ob.getPhysicalPath())[len(site_path):]

        uid = IUUID(ob)

        # first off, get url based off of public_url setting
        url = ob.absolute_url().replace(self.site.absolute_url(), self.public_url)

        resp = self.url_opener(url)
        if resp is None:
            return
        new_url = self.move_to_aws(self.massage_plone_resp_content(ob, resp, url),
                                   aws_content_path,
                                   content_type=resp['headers']['content-type'])

        view_url = None
        if ob.portal_type in self.view_url_types:
            aws_content_path += '/view'
            url += '/view'
            # first, download the regular url type for this content...
            resp = self.url_opener(url)
            if resp is not None:
                view_url = self.move_to_aws(self.massage_plone_resp_content(ob, resp, url),
                                            aws_content_path,
                                            content_type=resp['headers']['content-type'])

        self.archives[uid] = {
            'path': content_path,
            'url': new_url,
            'view_url_type': ob.portal_type in self.view_url_types,
            'view_url': view_url
        }
        self.path_to_uid[content_path] = uid
        return new_url

    def get_archive_url_by_uid(self, uid):
        return self.archives[uid]['url']

    def get_archive_url_by_path(self, path, wants_view=False):
        uid = self.path_to_uid[path]
        data = self.archives[uid]
        if wants_view:
            return data['view_url']
        else:
            return data['url']



# Dynamic fragment template support for simple tiles
from AccessControl import getSecurityManager
from AccessControl import Unauthorized
from castle.cms.fragments.interfaces import FRAGMENTS_DIRECTORY
from castle.cms.fragments.interfaces import IFragmentsDirectory
from plone import api
from plone.app.theming.interfaces import THEME_RESOURCE_NAME
from plone.app.theming.utils import theming_policy
from plone.registry.interfaces import IRegistry
from plone.resource.utils import queryResourceDirectory
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from zExceptions import NotFound
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implements
from zope.publisher.browser import BrowserPage

import logging
import os
import threading
import time


logger = logging.getLogger('castle')

_local_file_cache = threading.local()
_theme_file_cache = threading.local()


class FileCacheFactory(object):

    def get_cache_storage(self):
        try:
            cache = _local_file_cache.data
        except:
            _local_file_cache.data = {}
            cache = _local_file_cache.data
        return cache

    def get(self, name, filepath):
        cache = self.get_cache_storage()
        if filepath not in cache:
            if not os.path.exists(filepath):
                cache[filepath] = {
                    'template': None,
                    'mtime': 0
                }
            else:
                fi = open(filepath)
                cache[filepath] = {
                    'template': ZopePageTemplate(name, text=fi.read()),
                    'mtime': 0
                }
                fi.close()
        return cache[filepath]['template']


class FileChangedCacheFactory(FileCacheFactory):
    def get(self, name, filepath):
        cache = self.get_cache_storage()
        update = False
        if filepath not in cache:
            update = True
        else:
            data = cache[filepath]
            # check mtime
            if os.path.exists(filepath) and os.path.getmtime(filepath) > data['mtime']:
                # need to update, it changed
                update = True
        if update:
            if not os.path.exists(filepath):
                cache[filepath] = {
                    'template': None,
                    'mtime': 9999999999
                }
            else:
                fi = open(filepath)
                cache[filepath] = {
                    'template': ZopePageTemplate(name, fi.read()),
                    'mtime': os.path.getmtime(filepath)
                }
                fi.close()
        return cache[filepath]['template']


class FragmentsDirectory(object):
    implements(IFragmentsDirectory)

    order = 10
    layer = None

    directory_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        'templates'
    )

    def __init__(self):
        if api.env.debug_mode():
            self.cache = FileChangedCacheFactory()
        else:
            self.cache = FileCacheFactory()

    def template_factory(self, name, template):
        return ZopePageTemplate(name, text=template)

    def get(self, context, request, name):
        template_path = "%s/%s.pt" % (self.directory_path, name,)

        # Now disable the theme so we don't double-transform
        request.response.setHeader('X-Theme-Disabled', '1')

        template = self.cache.get(name, template_path)
        if template is None:
            raise KeyError(name)

        return FragmentView(
            context, request, name, 'View', template)

    def list(self):
        res = []
        if not os.path.exists(self.directory_path):
            return res
        for name in os.listdir(self.directory_path):
            if name.endswith('.pt') and name[0] != '.':
                res.append(name.replace('.pt', ''))
        return res


class ThemeFragmentsDirectory(object):
    implements(IFragmentsDirectory)

    order = 100
    layer = None

    def __init__(self):
        self.cache = {}

    def _reset_cache(self):
        _theme_file_cache.data = {
            'mtime': time.time(),
            'templates': {}
        }
        return _theme_file_cache.data

    def get_mtime(self, fi):
        try:
            return fi.lastModifiedTimestamp
        except (AttributeError, KeyError):
            return float(fi.bobobase_modification_time())

    def get_from_cache(self, policy, theme_directory, name, template_path):
        mtime = policy.getCacheStorage()['mtime']
        try:
            cache = _theme_file_cache.data
        except:
            cache = self._reset_cache()

        key = policy.getCacheKey() + template_path
        update = False
        if mtime > cache['mtime']:
            # was invalidated, we need to update the template
            cache = self._reset_cache()
            update = True
        elif key not in cache['templates']:
            update = True

        if not update and key in cache['templates'] and api.env.debug_mode():
            # check if we need to update the cache
            try:
                template = cache['templates'][key]
                if self.get_mtime(theme_directory[template_path]) > template['mtime']:
                    update = True
            except:
                raise KeyError(name)

        if update:
            # use theme cache setting to see if we should re-read
            if not theme_directory.isFile(str(template_path)):
                raise KeyError(name)
            fi = theme_directory[template_path]
            template = ZopePageTemplate(name, text=theme_directory.readFile(str(template_path)))
            cache['templates'][key] = {
                'template': template,
                'mtime': self.get_mtime(fi)
            }
        return cache['templates'][key]['template']

    def get(self, context, request, name):
        policy = theming_policy()
        theme_directory = self.get_directory(policy)

        template_path = "%s/%s.pt" % (FRAGMENTS_DIRECTORY, name)

        template = self.get_from_cache(policy, theme_directory, name, template_path)

        # Now disable the theme so we don't double-transform
        request.response.setHeader('X-Theme-Disabled', '1')

        return FragmentView(
            context, request, name, 'View', template)

    def get_directory(self, policy=None):
        if policy is None:
            policy = theming_policy()

        current_theme = policy.getCurrentTheme()
        if current_theme is None:
            raise KeyError()

        theme_directory = queryResourceDirectory(THEME_RESOURCE_NAME, current_theme)

        if theme_directory is None:
            raise KeyError()

        return theme_directory

    def list(self):
        themeDirectory = self.get_directory()
        try:
            frag_directory = themeDirectory[FRAGMENTS_DIRECTORY]
        except NotFound:
            return []
        res = []
        for name in frag_directory.listDirectory():
            if name.endswith('.pt') and name[0] != '.':
                res.append(name.replace('.pt', ''))
        return res


class FragmentView(BrowserPage):
    """View class for template-based views defined in the theme.
    When you traverse to ``..../@@fragment/foobar`` to render the view
    defined in ``fragments/foobar.pt`` in the theme, this becomes the ``view``.
    """

    def __init__(self, context, request, name, permission, template):
        super(FragmentView, self).__init__(context, request)
        self.__name__ = name
        self._permission = permission
        self._template = template

    def __call__(self, options=None, *args, **kwargs):
        sm = getSecurityManager()
        if not sm.checkPermission(self._permission, self.context):
            raise Unauthorized()

        site = api.portal.get()
        site_url = site.absolute_url()
        registry = getUtility(IRegistry)
        public_url = registry.get('plone.public_url', None)
        if not public_url:
            public_url = site_url
            if not api.user.is_anonymous():
                site_url = public_url

        utils = getMultiAdapter((self.context, self.request),
                                name='castle-utils')
        boundNames = {
            'context': self.context,
            'request': self.request,
            'view': self,
            'portal_url': site_url,
            'public_url': public_url,
            'site_url': site_url,
            'registry': registry,
            'portal': site,
            'utils': utils
        }
        if options is not None:
            boundNames.update(options)

        zpt = self._template.__of__(self.context)
        try:
            return zpt._exec(boundNames, args, kwargs)
        except NotFound as e:
            # We don't want 404's for these - they are programming errors
            raise Exception(e)



from zope.annotation.interfaces import IAnnotations

PDF_KEY = 'castle.cms.pdf'
SCREENSHOT_KEY = 'castle.cms.pdfscreenshot'


class PDFSetting(object):
    def __init__(self, obj):
        self.obj = obj
        self.annotations = IAnnotations(obj)

    def get(self, default=None):
        if self.created:
            return self.annotations[PDF_KEY]
        return default

    def put(self, blob):
        self.annotations[PDF_KEY] = blob

    def put_screenshot(self, blob):
        self.annotations[SCREENSHOT_KEY] = blob

    def get_screenshot(self, default=None):
        if self.screenshot_created:
            return self.annotations[SCREENSHOT_KEY]
        return default

    @property
    def created(self):
        return PDF_KEY in self.annotations

    @property
    def screenshot_created(self):
        return SCREENSHOT_KEY in self.annotations



# -*- coding: utf-8 -*-
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from castle.cms.tests.utils import get_tile
from castle.cms.tests.utils import render_tile
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.uuid.interfaces import IUUID

import unittest


class TestTiles(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING
    prefix = 'castle.cms.'

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))

    def test_subscribe_render(self):

        name = self.prefix + 'subscription'
        data = {}

        page = render_tile(self.request, self.portal, name, data)

        # Not really much to test here?
        # Just make sure the form renders
        self.assertTrue('form.buttons.subscribe' in page)

    def test_facebook_render(self):

        name = self.prefix + 'facebookPage'
        data = {
            'href': u'https://www.facebook.com/plonecms/?fref=ts',
            'timeline': 'timeline'
        }
        page = render_tile(self.request, self.portal, name, data)
        self.assertTrue('data-tabs="timeline"' in page)
        self.assertTrue('data-href="https://www.facebook.com/plonecms"' in page)

    def test_sharebutton_render(self):

        name = self.prefix + 'sharing'
        data = {
            'buttons': ['twitter', 'facebook']
        }

        page = render_tile(self.request, self.portal, name, data)

        self.assertTrue('Share on Facebook' in page)
        self.assertTrue('Share on Twitter' in page)

    def test_pinterest_render(self):
        name = self.prefix + 'pin'
        data = {
            'url': u'https://www.pinterest.com/pin/546413367268498761/'
        }

        page = render_tile(self.request, self.portal, name, data)
        self.assertTrue('data-pin-do="embedPin"' in page)
        self.assertTrue('href="https://www.pinterest.com/pin/546413367268498761/"' in page)

    def test_timeline_render(self):

        name = self.prefix + 'twitterTimeline'
        data = {
            'screenName': u'plone',
            'theme': u'dark'
        }
        page = render_tile(self.request, self.portal, name, data)
        # Twitter WidgetID set by default
        self.assertTrue('684100313582833665' in page)
        # Checking for other simple configurations
        self.assertTrue('data-pat-timeline' in page)
        self.assertTrue('plone' in page)

    def test_tweet_render(self):

        name = self.prefix + 'tweet'
        data = {
            'url': u'https://twitter.com/xkcdComic/status/678810713033277440',
            'cards': 'hidden'
        }

        page = render_tile(self.request, self.portal, name, data)
        # Pretty much just assuring it renders correctly.
        # Not much else to do here
        self.assertTrue('678810713033277440' in page)
        self.assertTrue('data-pat-tweet' in page)
        self.assertTrue('hidden' in page)

    def test_video_render(self):
        name = self.prefix + 'videotile'
        data = {
            'youtube_url': u'https://www.youtube.com/watch?v=wZZ7oFKsKzY'
        }

        page = render_tile(self.request, self.portal, name, data)
        self.assertTrue('https://www.youtube-nocookie.com/embed/wZZ7oFKsKzY' in page)
        self.assertTrue('iframe' in page)

    def test_map_render(self):
        name = self.prefix + 'maptile'
        data = {
            'center': '{"formatted":"Oshkosh, WI, USA","lat":44.0247062,"lng":-88.54261359999998}'
        }

        page = render_tile(self.request, self.portal, name, data)
        self.assertTrue('class="map-container"' in page)
        self.assertTrue('Oshkosh, WI' in page)

    def test_calendar_render(self):
        name = self.prefix + 'calendar'
        data = {}

        # create an event for today to make sure the calendar reflects
        # current events
        api.content.create(type='Event', id='event1', container=self.portal)

        page = render_tile(self.request, self.portal, name, data)
        self.assertTrue('data-pat-fullcalendar=' in page)
        # make sure there's a link to the event in the calendar
        self.assertTrue('event1' in page)

    def test_embed_render(self):
        embedCode = '<iframe width="560" height="315" src="https://www.youtube.com/embed/JbpgM-JTang" frameborder="0" allowfullscreen></iframe>'  # noqa

        name = self.prefix + 'embedtile'
        data = {
            'code': embedCode
        }

        page = render_tile(self.request, self.portal, name, data)

        # Embed tile just spits the embed code back out onto the page.
        self.assertTrue(embedCode in page)

    def test_querylisting_results(self):
        api.content.create(type='Document', id='page1', container=self.portal,
                           subject=('foobar',), title='Foobar')
        api.content.create(type='Document', id='page2', container=self.portal,
                           subject=('foobar',), title='Foobar')
        api.content.create(type='Document', id='page3', container=self.portal,
                           subject=('foobar', 'foobar2'), title='Foobar')
        api.content.create(type='Document', id='page4', container=self.portal,
                           subject=('foobar', 'foobar2'), title='Foobar')
        data = {
            'query': [{
                'i': 'Subject',
                'o': 'plone.app.querystring.operation.list.contains',
                'v': 'foobar2'
            }],
            'available_tags': ('foobar2',)
        }
        tile = get_tile(self.request, self.portal, 'castle.cms.querylisting', data)
        self.assertEqual(tile.results()['total'], 2)

        self.request.form.update({
            'Subject': 'foobar'
        })
        tile = get_tile(self.request, self.portal, 'castle.cms.querylisting', data)
        self.assertEqual(tile.results()['total'], 4)

        name = self.prefix + 'querylisting'
        page = render_tile(self.request, self.portal, name, data)
        self.assertTrue('Foobar' in page)

    def test_existing_content_tile(self):
        page = api.content.create(
            type='Document', id='page1', container=self.portal,
            subject=('foobar',), title='Foobar', description='Some foobar stuff')
        data = {
            'content': [IUUID(page)]
        }
        name = self.prefix + 'existing'
        html = render_tile(self.request, page, name, data)
        self.assertTrue('existing-content-basic' in html)

        data['display_type'] = 'backgroundimage'
        html = render_tile(self.request, page, name, data)
        self.assertTrue('existing-content-backgroundimage' in html)



from AccessControl.PermissionRole import rolesForPermissionOn
from Acquisition import aq_parent
from castle.cms import archival
from castle.cms.cron.utils import login_as_admin
from castle.cms.cron.utils import setup_site
from castle.cms.cron.utils import spoof_request
from castle.cms.utils import get_backend_url
from castle.cms.utils import retriable
from castle.cms.utils import send_email
from plone import api
from plone.app.layout.navigation.defaultpage import getDefaultPage
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from tendo import singleton

import logging
import transaction


logger = logging.getLogger('castle.cms')


@retriable(sync=True)
def archive(site):
    setup_site(site)

    if (not api.portal.get_registry_record('castle.archival_enabled') or
            not api.portal.get_registry_record('castle.aws_s3_bucket_name') or
            not api.portal.get_registry_record('castle.aws_s3_key') or
            not api.portal.get_registry_record('castle.aws_s3_secret') or
            not api.portal.get_registry_record('plone.public_url')):
        logger.error('Can not archive content. Either not enabled, S3 API not set or no public '
                     'url set')
        return

    storage = archival.Storage(site)
    for brain in archival.getContentToArchive():
        try:
            ob = brain.getObject()

            container = aq_parent(ob)
            if (IPloneSiteRoot.providedBy(container) and
                    getDefaultPage(container) == ob.getId()):
                continue

            allowed = set(rolesForPermissionOn('View', ob))
            if 'Anonymous' not in allowed:
                # we can *not* archive unpublished content
                continue
            new_url = storage.add_content(ob)

            # resets login creds..
            login_as_admin(app)  # noqa

            if new_url:
                logger.warn('imported %s -> %s' % (ob.absolute_url(), new_url))
                # XXX might need to re-architect... might get conflict errors with how slow
                # archiving takes...
                api.content.delete(ob)
                transaction.commit()
            else:
                logger.error('error importing %s' % ob.absolute_url())
        except:
            logger.error('Error archiving %s' % brain.getPath(), exc_info=True)

    content_to_archive = archival.getContentToArchive(7)
    if len(content_to_archive) == 0:
        return

    backend_url = get_backend_url()
    # send out email warning of content about to be archived
    email_text = """
<p>Warning, this content will be archived in 7 days.
Login to
<a href="{site_url}">{site_title}</a> to extend this content.
</p>
<ul>""".format(
        site_title=api.portal.get_registry_record('plone.site_title'),
        site_url=backend_url)

    site_url = api.portal.get().absolute_url()
    for brain in content_to_archive:
        url = brain.getURL()
        url = url.replace(site_url, backend_url)
        email_text += """<li>
<a href="{url}">{title}</a></li>""".format(url=url, title=brain.Title)

    email_text += '</ul>'

    for user in api.user.get_users():
        roles = api.user.get_roles(user=user)
        if ('Site Administrator' not in roles and
                'Manager' not in roles):
            continue
        email = user.getProperty('email')
        if not email:
            continue

        name = user.getProperty('fullname') or user.getId()
        html = '<p>Hi {name},</p>'.format(name=name) + email_text
        send_email(
            recipients=email,
            subject="Content will be archived(Site: %s)" % (
                api.portal.get_registry_record('plone.site_title')),
            html=html)


def run(app):
    singleton.SingleInstance('archivecontent')

    app = spoof_request(app)  # noqa
    for oid in app.objectIds():  # noqa
        obj = app[oid]  # noqa
        if IPloneSiteRoot.providedBy(obj):
            try:
                archive(obj)
            except:
                logger.error('Could not archive %s' % oid, exc_info=True)


if __name__ == '__main__':
    run(app)  # noqa



from castle.cms.exportimport.frontpagecontent import getTileData
from castle.cms.interfaces import ICastleLayer
from castle.cms.interfaces import IGlobalTile
from castle.cms.tiles.meta import MetaTile
from plone.app.blocks.layoutbehavior import ILayoutAware
from plone.tiles.interfaces import IPersistentTile
from plone.tiles.interfaces import ITileDataManager
from Products.CMFPlone.interfaces import INonInstallable
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.interface import implements


class HiddenProducts(object):
    implements(INonInstallable)

    def getNonInstallableProducts(self):
        return []

    def getNonInstallableProfiles(self):
        return [
            'castle.theme:default',
            'plone.formwidget.querystring:default',
            'plone.session:default',
            'plone.app.multilingual:default',
            'Products.ATContentTypes:default',
            'archetypes.multilingual:default',
            'plone.app.standardtiles:default',
            'plone.app.tiles:default',
            'plone.app.mosaic:default',
            'plone.app.drafts:default',
            'plone.app.blocks:default',
            'collective.documentviewer:default',
            'collective.elasticsearch:default',
            'castle.cms:default',
            ]


FOOTER_HTML = """
<div class="row">
    <div class="col-sm-3">
        <h5>Connect with Us</h5>
        <ul class="footer-contact">
            <li><span class="icon icon-phone"></span><a href="tel:+17152526769">(715) 869-3440</a></li>
            <li><span class="icon icon-email"></span><a href="mailto:info@wildcardcorp.com">info@wildcardcorp.com</a></li>
            <li>
                <ul class="social-icons">
                    <li><a href="https://www.facebook.com/wildcardcorp"><span class="sr-only">Facebook</span><span class="icon icon-footer-fb"></span></a></li>
                    <li><a href="https://twitter.com/WildcardCorp"><span class="sr-only">Twitter</span><span class="icon icon-footer-twitter"></span></a></li>
                    <li><a href="https://www.linkedin.com/company/wildcard-corp-?trk=company_logo"><span class="sr-only">Linkedin</span><span class="icon icon-footer-linkedin"></span></a></li>
                </ul>
            </li>
        </ul>
    </div>
    <div class="col-sm-7 col-sm-offset-2">
        <h5>Get News & Updates</h5>
        <p>Sign up for product updates and the latest goings-ons from Wildcard Corp.</p>
        <form class="form-inline">
            <div class="form-group">
                <label class="sr-only" for="newsletter-email">Email Address</label>
                <div class="input-group">
                    <input type="email" class="form-control" id="newsletter-email" placeholder="Email Address">
                    <span class="input-group-btn">
                        <a href="#" class="btn btn-primary">Sign Up</a>
                    </span>
                </div>
            </div>
        </form>
    </div>
</div>"""


_tiles_data = {
    'footer': [{
        'data': {'content': FOOTER_HTML},
        'meta': {'type': 'plone.app.standardtiles.rawhtml', 'id': 'footer-html'}
    }]
}


def tiles(site, req, tiles_data=_tiles_data):

    alsoProvides(req, ICastleLayer)

    # Sitewide slot tiles
    for meta_id, tiles in tiles_data.items():
        meta_tile = MetaTile(site, req)
        alsoProvides(meta_tile, IGlobalTile)
        meta_tile.id = 'meta-' + meta_id
        meta_tile.__name__ = 'castle.cms.meta'
        meta_data_manager = ITileDataManager(meta_tile)
        meta_data = meta_data_manager.get()
        meta_tiles = []

        for tile_data in tiles:
            meta_tiles.append(tile_data['meta'])
            tile = getMultiAdapter((site, req), name=tile_data['meta']['type'])
            alsoProvides(tile, IPersistentTile)
            tile.id = tile_data['meta']['id']
            tile.__name__ = tile_data['meta']['type']
            data_manager = ITileDataManager(tile)
            data_manager.set(tile_data['data'])

        meta_data['tiles'] = meta_tiles
        meta_data_manager.set(meta_data)

    frontpage = site['front-page']
    adapted = ILayoutAware(frontpage, None)
    if adapted:
        adapted.pageSiteLayout = 'frontpage.html'

    # Tiles only for the front-page
    frontpage_tiles = getTileData()

    for tile in frontpage_tiles:
        fp_tile = getMultiAdapter((frontpage, req), name=tile['meta']['type'])

        meta = tile['meta']
        fp_tile.id = meta['id']
        alsoProvides(fp_tile, IPersistentTile)
        data_manager = ITileDataManager(fp_tile)
        data_manager.set(tile['data'])

        if 'slot' in tile:
            meta_tile = MetaTile(frontpage, req)
            alsoProvides(meta_tile, IGlobalTile)
            meta_tile.id = 'meta-' + tile['slot']
            meta_tile.__name__ = 'castle.cms.meta'
            meta_data_manager = ITileDataManager(meta_tile)
            meta_data = meta_data_manager.get()

            existing_tiles = meta_data.get('tiles') or []
            existing_tiles.append(meta)
            meta_data['tiles'] = existing_tiles
            meta_data['mode'] = 'show'

            meta_data_manager.set(meta_data)

    frontpageLayout = ILayoutAware(frontpage, None)
    if frontpageLayout:
        frontpageLayout.contentLayout = '/++contentlayout++castle/frontpage-layout.html'



from castle.cms._scripts.utils import get_js_resource_object
from castle.cms._scripts.utils import get_module_dir
from castle.cms._scripts.utils import get_url_resource_object
from castle.cms._scripts.utils import recursive_resources
from castle.cms._scripts.utils import remove_extension
from castle.cms._scripts.utils import resource_to_path, get_resource_object
from castle.cms.cron.utils import get_sites
from castle.cms.cron.utils import setup_site
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import IBundleRegistry
from Products.CMFPlone.interfaces import IResourceRegistry
from zope.component import getUtility

import castle.cms
import json
import mockup
import os
import plone.app.mosaic
import plone.app.portlets
import plonetheme.barceloneta
import Products.CMFPlone


DIR = os.path.dirname(os.path.realpath(__file__))
WORK_DIR = os.path.join(os.getcwd(), 'devtools')


if not os.path.exists(WORK_DIR):
    os.mkdir(WORK_DIR)


def jdump(data):
    return json.dumps(data, sort_keys=True,
                      indent=4, separators=(',', ': '))


def add_quotes(ss):
    return '"' + ss + '"'


SCRIPT_DIR = os.path.join(get_module_dir(castle.cms), '_scripts')
CMFPlone_DIR = get_module_dir(Products.CMFPlone)

webpack_aliases = {}
bundles_config = {}


for site in get_sites(app):  # noqa
    setup_site(site)
    os.environ['SITE_ID'] = site.getId()
    break


registry = getUtility(IRegistry)
bundles = registry.collectionOfInterface(
    IBundleRegistry, prefix="plone.bundles", check=False)
resources = registry.collectionOfInterface(
    IResourceRegistry, prefix="plone.resources", check=False)


for rkey, resource in resources.items():
    js_object = get_js_resource_object(site, resource)
    if js_object:
        webpack_aliases[rkey] = remove_extension(resource_to_path(js_object))
    url_object = get_url_resource_object(site, resource)
    if url_object:
        fs_path = resource_to_path(url_object)
        webpack_aliases[rkey + '-url'] = fs_path
        webpack_aliases.update(recursive_resources(rkey + '-url', fs_path))
    for css in resource.css:
        css_obj = get_resource_object(site, css)
        if css_obj:
            if '.less' in css:
                webpack_aliases[rkey + '.less'] = resource_to_path(css_obj)
            else:
                webpack_aliases[rkey + '.css'] = resource_to_path(css_obj)


for bkey, bundle in bundles.items():
    if bundle.compile:
        js_requires = []
        css_requires = []
        build_filename = os.path.join(WORK_DIR, '{}-build.js'.format(bkey))
        less_build_filename = os.path.join(WORK_DIR, '{}-build.less'.format(bkey))
        for resource_key in bundle.resources:
            js_object = get_js_resource_object(site, resources[resource_key])
            if js_object:
                bfilepath = resource_to_path(js_object)
                js_requires.append(bfilepath)
            for css in resources[resource_key].css:
                css_obj = get_resource_object(site, css)
                css_requires.append(resource_to_path(css_obj))
        bundles_config[bkey] = build_filename
        with open(build_filename, 'w') as fi:
            fi.write('\n'.join(['require("' + f + '");' for f in js_requires]))
        with open(less_build_filename, 'w') as fi:
            fi.write('@import "{}/variables.less";\n'.format(WORK_DIR))
            for css_require in css_requires:
                # the less plugin doesn't like absolute paths? wahhhhh?
                parts = css_require.split('/')
                filename = parts[-1]
                alias_name = '{}-{}-dir'.format(bkey, filename.replace('.', '-'))
                webpack_aliases[alias_name] = '/'.join(parts[:-1])
                fi.write('@import "{}/{}";\n'.format(webpack_aliases[alias_name], filename))


webpack_aliases.update({
    'webpack-build': WORK_DIR,
    'bower': os.path.join(CMFPlone_DIR, 'static/components'),
    'mockup': os.path.join(get_module_dir(mockup), 'patterns'),
    'plone': os.path.join(CMFPlone_DIR, 'static'),
    'mockup-less': os.path.join(get_module_dir(mockup), 'less')
})
webpack_config_data = {
    'cmfplone': CMFPlone_DIR,
    'static': os.path.join(CMFPlone_DIR, 'static'),
    'bower': os.path.join(CMFPlone_DIR, 'static/components'),
    'mockup': get_module_dir(mockup),
    'mosaic': get_module_dir(plone.app.mosaic),
    'barceloneta': get_module_dir(plonetheme.barceloneta),
    'castle': os.path.join(get_module_dir(castle.cms), 'static'),
    'aliases': jdump(webpack_aliases)
}
less_config = {
    'globalVars': {
        'bowerPath': add_quotes(webpack_aliases['bower'] + '/'),
        'mockupPath': add_quotes(webpack_aliases['mockup']),
        'staticPath': add_quotes(webpack_aliases['plone']),
        'fontsPrefix': add_quotes(webpack_aliases['plone'] + '/fonts/'),
        'mockuplessPath': add_quotes(webpack_aliases['mockup-less'])
    }
}

with open(os.path.join(SCRIPT_DIR, 'templates/variables.less')) as fi:
    tmpl = fi.read()
    with open(os.path.join(WORK_DIR, 'variables.less'), 'w') as output:
        output.write(tmpl % webpack_aliases)


with open(os.path.join(SCRIPT_DIR, 'templates/webpack.globals.js')) as fi:
    tmpl = fi.read()
    with open(os.path.join(WORK_DIR, 'webpack.globals.js'), 'w') as output:
        output.write(tmpl % webpack_config_data)


with open(os.path.join(SCRIPT_DIR, 'templates/watchable-grunt.js')) as fi:
    tmpl = fi.read()
    with open('watchable-grunt.js', 'w') as output:
        output.write(tmpl % webpack_config_data)


with open(os.path.join(SCRIPT_DIR, 'templates/webpack.config.js')) as fi:
    tmpl = fi.read()
    with open('webpack.config.js', 'w') as output:
        output.write(tmpl % {
            'bundles_config': jdump(bundles_config),
            'bundles_keys': jdump(bundles_config.keys()),
            'common': 'common',

            # XXX not currently used
            'less_globals': jdump(less_config),
            'output': jdump({
                "pathinfo": True,
                "path": os.path.join(get_module_dir(castle.cms), 'static'),
                "filename": '[name]-compiled.min.js',
                "sourceMapFilename": '[name].map'
            })
        })


with open(os.path.join(SCRIPT_DIR, 'templates/package.json')) as fi:
    tmpl = fi.read()
    with open('package.json', 'w') as output:
        output.write(tmpl)


with open(os.path.join(SCRIPT_DIR, 'templates/Makefile')) as fi:
    tmpl = fi.read()
    with open('Makefile', 'w') as output:
        output.write(tmpl)

print('generate original grunt file')

with open(os.path.join(CMFPlone_DIR, '_scripts/_generate_gruntfile.py')) as fi:
    code = fi.read()
    exec(code, globals(), locals())
# __import__('Products.CMFPlone._scripts._generate_gruntfile', globals(), locals())



from AccessControl import getSecurityManager
from Acquisition import aq_base
from Acquisition import aq_parent
from castle.cms import cache
from castle.cms import utils
from castle.cms.behaviors.location import ILocation
from castle.cms.browser.nextprev import NextPrevious
from castle.cms.interfaces import IUtils
from castle.cms.vocabularies import LocationsVocabulary
from DateTime import DateTime
from datetime import datetime
from lxml import etree
from lxml.html import tostring
from plone import api
from plone.app.imaging.utils import getAllowedSizes
from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.navigation.defaultpage import getDefaultPage
from plone.app.layout.viewlets.common import GlobalSectionsViewlet
from plone.app.uuid.utils import uuidToObject
from plone.event.utils import pydt
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize.view import memoize
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.resources import add_resource_on_request
from Products.CMFPlone.utils import getSiteLogo
from Products.Five import BrowserView
from Products.ZCatalog.interfaces import ICatalogBrain
from unidecode import unidecode
from urlparse import parse_qs
from urlparse import urlparse
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import implements
from zope.viewlet.interfaces import IViewlet
from zope.viewlet.interfaces import IViewletManager

import json


def _one(val):
    if type(val) in (list, tuple, set) and len(val) > 0:
        return val[0]


def _clean_youtube_id(val):
    return val.split('&')[0]


class Utils(BrowserView):
    implements(IUtils)

    @property
    @memoize
    def registry(self):
        return getUtility(IRegistry)

    @property
    @memoize
    def site(self):
        return api.portal.get()

    @property
    @memoize
    def image_size(self):
        return getAllowedSizes()

    @property
    def types_use_view_action(self):
        return frozenset(
            self.registry.get('plone.types_use_view_action_in_listings', []))

    def get_registry_value(self, name, default=None):
        return self.registry.get(name, default)

    def get_object_url(self, obj):
        if obj is None:
            return
        try:
            url = obj.getURL()
        except AttributeError:
            url = obj.absolute_url()
        if obj.portal_type in self.types_use_view_action:
            url += '/view'
        return url

    def get_object(self, val):
        if ICatalogBrain.providedBy(val):
            return val.getObject()
        if not val:
            return None
        if isinstance(val, basestring):
            if val[0] == '/':
                # it's a path
                return self.site.restrictedTraverse(val.strip('/'), None)
            else:
                # try querying catalog
                obj = uuidToObject(val)
                if obj:
                    return obj
        if isinstance(val, basestring):
            return None
        return val

    def has_image(self, obj):
        return utils.has_image(obj)

    def get_path(self, obj):
        return utils.get_path(obj)

    def valid_date(self, date):
        if not date:
            return False
        if isinstance(date, basestring):
            date = DateTime(date)
        if date.year() == 1969:
            return False
        return True

    def iso_date(self, date):
        if not date:
            return ''
        if isinstance(date, basestring):
            date = DateTime(date)
        return date.ISO8601()

    def format_date(self, date, format='common', formatter=None):
        if not date:
            return ''
        if isinstance(date, basestring):
            date = DateTime(date)
        if isinstance(date, datetime):
            date = DateTime(date)

        if formatter is not None:
            try:
                return date.strftime(formatter)
            except:
                pass

        if format == 'common':
            return date.fCommon()
        if format in ('dot', 'dotshort'):
            return '%s.%s.%s' % (
                str(date.year()),
                str(date.month()).zfill(2),
                str(date.day()).zfill(2))
        if format == 'usdot':
            return '%s.%s.%s' % (
                str(date.month()).zfill(2),
                str(date.day()).zfill(2),
                str(date.year()),)
        if format == 'notime':
            return ' '.join(date.fCommon().split(' ')[:-2])
        if format == 'slash':
            return '%s/%s/%s' % (
                str(date.year()),
                str(date.month()).zfill(2),
                str(date.day()).zfill(2))
        if format == 'usslash':
            return '%s/%s/%s' % (
                str(date.month()).zfill(2),
                str(date.day()).zfill(2),
                str(date.year()),)

    def get_locations(self, obj):
        bdata = ILocation(obj, None)
        result = []
        if bdata and bdata.locations:
            vocab = LocationsVocabulary(self.context)
            for loc in bdata.locations:
                term = vocab.by_token.get(loc)
                if term:
                    result.append(term.title)
                else:
                    result.append(loc)
        return result

    def get_location(self, obj):
        try:
            location = obj.location
            if location:
                return location
        except AttributeError:
            pass

        if ICatalogBrain.providedBy(obj):
            obj = obj.getObject()

        bdata = ILocation(obj, None)
        if bdata and bdata.locations:
            vocab = LocationsVocabulary(self.context)
            term = vocab.by_token.get(bdata.locations[0])
            if term:
                return term.title
            else:
                return bdata.locations[0]

    def clean_youtube_url(self, url):
        parsed = urlparse(url)
        if parsed.query:
            query = parse_qs(parsed.query)
            if 'v' in query:
                return 'https://www.youtube-nocookie.com/embed/%s' % _one(query['v'])
        if '/embed/' in url:
            _id = _clean_youtube_id(url.partition('/embed/')[-1])
            return 'https://www.youtube-nocookie.com/embed/%s' % _id
        if 'youtu.be' in url:
            _id = _clean_youtube_id(url.partition('https://youtu.be/')[-1])
            return 'https://www.youtube-nocookie.com/embed/%s' % _id
        if 'youtube' in url and '/v/' in url:
            _id = _clean_youtube_id(url.partition('/v/')[-1])
            return 'https://www.youtube-nocookie.com/embed/%s' % _id
        return url

    def get_youtube_url(self, obj):
        if isinstance(obj, basestring):
            url = obj
        else:
            try:
                url = obj.youtube_url
            except AttributeError:
                return

        if url is None:
            return

        return self.clean_youtube_url(url)

    def get_uid(self, obj):
        return IUUID(obj, None)

    def get_scale_url(self, content, scale='large'):
        return '%s/@@images/image/%s' % (content.absolute_url(), scale)

    def get_main_links(self):
        viewlet = GlobalSectionsViewlet(self.context, self.request, None)
        viewlet.update()
        selected_tab = viewlet.selected_portal_tab

        site = api.portal.get()
        site_default_page = getDefaultPage(site)
        if selected_tab == site_default_page:
            selected_tab = 'index_html'

        default_page_url = '{}/{}'.format(site.absolute_url(), site_default_page)
        tabs = []
        for tab in viewlet.portal_tabs:
            if tab['url'] == default_page_url:
                continue
            tabs.append(tab)

        return {
            'selected_portal_tab': selected_tab,
            'portal_tabs': tabs
        }

    def get_actions(self, name):
        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')
        return context_state.actions(name)

    def get_folder_section(self):
        section = self.context
        while True:
            parent = aq_parent(section)
            if IPloneSiteRoot.providedBy(parent) or parent is None:
                break
            section = parent
        sm = getSecurityManager()
        if not sm.checkPermission('View', section):
            return
        return section

    def get_public_url(self):
        return utils.get_public_url()

    def render_breadcrumbs(self):
        alsoProvides(self, IViewView)
        manager = queryMultiAdapter(
            (self.context, self.request, self),
            IViewletManager, name='plone.abovecontent'
        )
        viewlet = queryMultiAdapter(
            (self.context, self.request, self, manager),
            IViewlet, name='plone.path_bar'
        )
        if viewlet is not None:
            viewlet.update()
            return viewlet.render()
        else:
            return ''

    def search(self, **kwargs):
        catalog = api.portal.get_tool('portal_catalog')
        return catalog(**kwargs)

    def is_anonymous(self):
        return api.user.is_anonymous()

    def truncate_text(self, *args, **kwargs):
        return utils.truncate_text(*args, **kwargs)

    def get_popular_tags(self, limit=20):
        cache_key = '-'.join(self.site.getPhysicalPath()[1:]) + '-popular-tags-' + str(limit)

        try:
            result = cache.get(cache_key)
        except:
            result = None
        if result is not None:
            return result

        cat = api.portal.get_tool('portal_catalog')
        index = cat._catalog.getIndex('Subject')
        tags = []
        for name in index._index.keys():
            try:
                number_of_entries = len(index._index[name])
            except TypeError:
                continue
            tags.append({
                'name': name,
                'count': number_of_entries
            })

        sorted_tags = list(reversed(sorted(tags, key=lambda tag: tag['count'])))[:limit]

        cache.set(cache_key, sorted_tags, 60 * 5)
        return sorted_tags

    def focal_image_tag(self, brain, scale=None, className='', imageClassName='',
                        attributes=None, focal=None):
        # read https://github.com/jonom/jquery-focuspoint on how to calc
        image_info = utils.get_image_info(brain)

        attrib = {}
        if attributes is not None:
            attrib.update(attributes)

        if 'src' not in attrib:
            try:
                url = brain.getURL()
                alt = brain.Title
            except:
                url = brain.absolute_url()
                alt = brain.Title()
            if not isinstance(alt, basestring):
                alt = ''
            attrib.update({
                'src': '{0}/@@images/image/{1}'.format(url, scale or ''),
                'alt': unidecode(alt),
                'class': imageClassName
            })

        el = etree.Element('div')
        el.attrib['class'] = 'focuspoint ' + className
        imEl = etree.Element('img')
        imEl.attrib.update(attrib)
        el.append(imEl)

        if not focal and image_info and 'focal_point' in image_info:
            focal = image_info['focal_point']

        if image_info and focal:
            width = image_info['width']
            height = image_info['height']
            focal_x = ((float(focal[0]) - (float(width) / 2)) / float(width)) * 2
            focal_y = -((float(focal[1]) - (float(height) / 2)) / float(height)) * 2

            sizes_info = {}
            for scale_name, scale_data in self.image_size.items():
                scale_ratio = float(scale_data[0]) / float(width)
                sizes_info[scale_name] = {
                    'w': int(float(width) * float(scale_ratio)),
                    'h': int(float(height) * float(scale_ratio))
                }

            el.attrib.update({
                'data-focus-x': str(focal_x),
                'data-focus-y': str(focal_y),
                'data-base-url': '{0}/@@images/image/'.format(url),
                'data-scale': scale,
                'data-w': str(width),
                'data-h': str(height),
                'data-scales-info': json.dumps(sizes_info)
            })

        return tostring(el)

    @property
    @memoize
    def normalizer(self):
        return getUtility(IIDNormalizer)

    def normalize(self, val):
        return self.normalizer.normalize(val)

    def get_logo(self):
        return getSiteLogo()

    def get_next_prev(self, obj):
        try:
            nextprev = NextPrevious(obj)
            return {
                'next': nextprev.getNextItem(obj),
                'prev': nextprev.getPreviousItem(obj)
            }
        except ValueError:
            return {
                'next': None,
                'prev': None
            }

    def get_summary_text(self, obj):
        obj = aq_base(obj)
        try:
            return obj.overview.output
        except:
            try:
                desc = obj.Description
                if callable(desc):
                    desc = desc()
                return desc
            except:
                pass
        return ''

    def add_resource_on_request(self, resource):
        add_resource_on_request(self.request, resource)



from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.WorkflowCore import WorkflowException
from Acquisition import aq_inner
from Products.CMFPlone import utils
from Products.CMFPlone.browser.interfaces import INavigationBreadcrumbs
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs
from Products.Five import BrowserView
from zope.interface import implementer
from plone import api
from castle.cms.browser.utils import Utils


@implementer(INavigationBreadcrumbs)
class PhysicalNavigationBreadcrumbs(BrowserView):

    def breadcrumbs(self):
        context = aq_inner(self.context)

        view_utils = Utils(self.context, self.request)

        crumbs = []
        while context is not None:
            if ISiteRoot.providedBy(context):
                break

            if utils.isDefaultPage(context, self.request):
                context = utils.parent(context)
                continue

            # Some things want to be hidden from the breadcrumbs
            if IHideFromBreadcrumbs.providedBy(context):
                context = utils.parent(context)
                continue

            item_url = view_utils.get_object_url(context)
            try:
                state = api.content.get_state(obj=context)
            except WorkflowException:
                state = None
            label = getattr(context, 'navigation_label', None)
            if not label:
                label = utils.pretty_title_or_id(context, context)
            crumbs.append({
                'absolute_url': item_url,
                'Title': label,
                'state': state
            })
            context = utils.parent(context)

        return list(reversed(crumbs))



try:
    import json
except ImportError:
    import simplejson as json

from datetime import datetime
from DateTime import DateTime
from StringIO import StringIO
import base64
from persistent.dict import PersistentDict
from Persistence.mapping import PersistentMapping as PM1
from persistent.mapping import PersistentMapping as PM2
from persistent.list import PersistentList
from BTrees.OOBTree import OOBTree
import re
from zope.dottedname.resolve import resolve
from ZPublisher.HTTPRequest import record
from ZODB.blob import Blob
from plone.app.blob.utils import openBlob
from Products.ZCatalog.Lazy import LazyCat
try:
    from plone.app.contentlisting.contentlisting import ContentListing
except:
    ContentListing = object()

from plone.app.blob.field import BlobWrapper
import OFS


_filedata_marker = 'filedata://'
_deferred_marker = 'deferred://'
_type_marker = 'type://'
_uid_marker = 'uid://'
_uid_separator = '||||'
_date_re = re.compile('^[0-9]{4}\-[0-9]{2}\-[0-9]{2}.*$')


class BaseTypeSerializer(object):
    klass = None
    toklass = None
    @classmethod
    def getTypeName(kls):
        return "%s.%s" % (kls.klass.__module__, kls.klass.__name__)
    @classmethod
    def serialize(kls, obj):
        if hasattr(obj, 'aq_base'):
            obj = obj.aq_base
        data = kls._serialize(obj)
        results = {
            'type': kls.getTypeName(),
            'data': data
        }
        return _type_marker + dumps(results)
    @classmethod
    def _serialize(kls, obj):
        return kls.toklass(obj)
    @classmethod
    def deserialize(kls, data):
        return kls._deserialize(data)
    @classmethod
    def _deserialize(kls, data):
        return kls.klass(data)


class PM1Serializer(BaseTypeSerializer):
    klass = PM1
    toklass = dict


class PM2Serializer(PM1Serializer):
    klass = PM2


class PersistentDictSerializer(BaseTypeSerializer):
    klass = PersistentDict
    toklass = dict


class OOBTreeSerializer(BaseTypeSerializer):
    klass = OOBTree
    toklass = dict


class PersistentListSerializer(BaseTypeSerializer):
    klass = PersistentList
    toklass = list


class setSerializer(BaseTypeSerializer):
    klass = set
    toklass = list


class OFSFileSerializer(BaseTypeSerializer):
    klass = OFS.Image.File
    @classmethod
    def _serialize(kls, obj):
        try:
            data = str(obj.data)
        except:
            data = str(obj.data.data)
        return {
            'data': base64.b64encode(data),
            'id': obj.id(),
            'title': obj.title,
            'content_type': obj.content_type
        }
    @classmethod
    def _deserialize(kls, data):
        file = base64.b64decode(data['data'])
        id = data['id']
        title = data['title']
        ct = data['content_type']
        return kls.klass(id, title, file, ct)


class BlobSerializer(BaseTypeSerializer):
    klass = Blob
    @classmethod
    def _serialize(kls, obj):
        blobfi = openBlob(obj)
        data = blobfi.read()
        blobfi.close()
        return {'data': base64.b64encode(data)}
    @classmethod
    def _deserialize(kls, data):
        blob = Blob()
        bfile = blob.open('w')
        data = base64.b64decode(data['data'])
        bfile.write(data)
        bfile.close()
        return blob


class OFSImageSerializer(OFSFileSerializer):
    klass = OFS.Image.Image


class DateTimeSerializer(BaseTypeSerializer):
    klass = DateTime
    @classmethod
    def getTypeName(kls):
        return 'DateTime.DateTime'
    @classmethod
    def _serialize(kls, obj):
        return obj.ISO8601()
    @classmethod
    def _deserialize(kls, data):
        return DateTime(data)


class datetimeSerializer(BaseTypeSerializer):
    klass = datetime
    @classmethod
    def _serialize(kls, obj):
        return obj.isoformat()
    @classmethod
    def _deserialize(kls, data):
        return datetime.strptime(data, '%Y-%m-%dT%H:%M:%S.%f')


class recordSerializer(BaseTypeSerializer):
    klass = record
    toklass = dict
    @classmethod
    def _deserialize(kls, data):
        rec = record()
        for key, value in data.items():
            setattr(rec, key, value)
        return rec


class ContentListingSerializer(BaseTypeSerializer):
    klass = ContentListing
    toklass = list
    @classmethod
    def _serialize(kls, obj):
        return list(obj._basesequence)
    @classmethod
    def _deserialize(kls, data):
        return LazyCat(data)


class BlobWrapperSerializer(BaseTypeSerializer):
    klass = BlobWrapper
    @classmethod
    def _serialize(kls, obj):
        blob = obj.getBlob()
        blobfi = openBlob(blob)
        data = blobfi.read()
        blobfi.close()
        return {
            'data': base64.b64encode(data),
            'filename': obj.getFilename()}
    @classmethod
    def _deserialize(kls, data):
        io = StringIO(data['data'])
        io.filename = data['filename']
        return io

_serializers = {
    PM1: PM1Serializer,
    PM2: PM2Serializer,
    PersistentDict: PersistentDictSerializer,
    OOBTree: OOBTreeSerializer,
    PersistentList: PersistentListSerializer,
    set: setSerializer,
    OFS.Image.Image: OFSImageSerializer,
    OFS.Image.File: OFSFileSerializer,
    DateTime: DateTimeSerializer,
    datetime: datetimeSerializer,
    record: recordSerializer,
    Blob: BlobSerializer,
    ContentListing: ContentListingSerializer,
    BlobWrapper: BlobWrapperSerializer
}


class Deferred:
    pass


def customhandler(obj):
    if hasattr(obj, 'aq_base'):
        obj = obj.aq_base
    _type = type(obj)
    if _type.__name__ == 'instance':
        _type = obj.__class__
    if _type in _serializers:
        serializer = _serializers[_type]
        return serializer.serialize(obj)
    else:
        return None
    return obj


def decodeUid(v):
    v = v[len(_uid_marker):]
    return v.split(_uid_separator)


def custom_decoder(d):
    if isinstance(d, list):
        pairs = enumerate(d)
    elif isinstance(d, dict):
        pairs = d.items()
    result = []
    for k, v in pairs:
        if isinstance(v, basestring):
            if v.startswith(_filedata_marker):
                if v == _filedata_marker + _deferred_marker:
                    v = Deferred
                else:
                    filedata = v[len(_filedata_marker):]
                    if filedata:
                        v = StringIO(base64.b64decode(filedata))
                    else:
                        v = ''
            elif v.startswith(_type_marker):
                v = v[len(_type_marker):]
                results = loads(v)
                try:
                    _type = resolve(results['type'])
                    serializer = _serializers[_type]
                    v = serializer.deserialize(results['data'])
                except ImportError:
                    pass
        elif isinstance(v, (dict, list)):
            v = custom_decoder(v)
        result.append((k, v))
    if isinstance(d, list):
        return [x[1] for x in result]
    elif isinstance(d, dict):
        return dict(result)


def loads(data):
    return json.loads(data, object_hook=custom_decoder)


def dumps(data):
    return json.dumps(data, default=customhandler)



# -*- coding: utf-8 -*-
from castle.cms.browser.sitemap import SiteMapView
from castle.cms.testing import CASTLE_PLONE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME

import unittest


class TestSiteMap(unittest.TestCase):

    layer = CASTLE_PLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ('Member', 'Manager'))
        if 'front-page' not in self.portal:
            self.page = api.content.create(type='Document', id='front-page',
                                           container=self.portal)
            self.portal.setDefaultPage('front-page')

    def test_get_objects(self):
        site_size = len(self.portal.portal_catalog())
        view = SiteMapView(self.portal, self.request)
        paths = [o for o in view.objects()]
        self.assertEquals(len(paths), site_size)
        self.assertEquals(paths[0]['loc'], self.portal.absolute_url())



from castle.cms import archival
from castle.cms.files import aws
from plone import api
from Products.Five import BrowserView
from zExceptions import Redirect
from zope.component import queryMultiAdapter


class NotFoundView(BrowserView):

    def __call__(self):
        self.notfound = self.context
        self.context = api.portal.get()
        archive_storage = archival.Storage(self.context)
        site_url = self.context.absolute_url()
        path = self.request.ACTUAL_URL[len(site_url):].rstrip('/')

        wants_view = False
        if path.endswith('/view'):
            wants_view = True
            path = path.rsplit('/view', 1)[0]

        new_url = None
        if path.startswith('/resolveuid'):
            uid = path.replace('/resolveuid/', '')
            try:
                new_url = archive_storage.get_archive_url_by_uid(uid)
            except:
                pass
        else:
            try:
                new_url = archive_storage.get_archive_url_by_path(path, wants_view)
            except:
                pass
        if new_url:
            # XXX need to force redirect this way since normal redirect
            # gets overridden with 404
            if self.request.environ.get('QUERY_STRING'):
                new_url += '?' + self.request.environ['QUERY_STRING']
            raise Redirect(aws.swap_url(new_url))

        # seems this overrides plone.app.redirector handler
        redirector = queryMultiAdapter((self.context, self.request),
                                       name=u'plone_redirector_view')
        if redirector:
            redirector.attempt_redirect()

        return self.index()



from castle.cms.interfaces import ISecuritySchema
from castle.cms.widgets import SelectFieldWidget
from plone.app.registry.browser import controlpanel
from Products.CMFPlone.controlpanel.browser import security


class SecurityControlPanelForm(security.SecurityControlPanelForm):
    schema = ISecuritySchema

    def updateFields(self):
        super(SecurityControlPanelForm, self).updateFields()
        self.fields['restrict_logins_to_countries'].widgetFactory = SelectFieldWidget


class SecurityControlPanel(controlpanel.ControlPanelFormWrapper):
    form = SecurityControlPanelForm



from castle.cms.tiles.content import ContentTile
from castle.cms.widgets import AudioRelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zope import schema
from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class AudioTile(ContentTile):
    default_display_fields = ()

    def render(self):
        return self.index()

    @property
    def audios(self):
        res = []
        audios = self.data.get('audio_files', [])
        if audios:
            for obj in audios:
                obj = self.utils.get_object(obj)
                try:
                    fi = obj.file
                except AttributeError:
                    continue
                res.append(obj)
        return res

    def get_url(self, audio):
        fi = audio.file
        return '%s/@@download/file/%s' % (audio.absolute_url(), fi.filename)

    def get_content_type(self, audio):
        fi = audio.file
        return fi.contentType


class IAudioTileSchema(model.Schema):

    form.widget(audio_files=AudioRelatedItemsFieldWidget)
    audio_files = schema.List(
        title=u"Audio files",
        description=u"Reference a files on the site. You can provide more "
                    u"than one audio file in the case you'd like to provide "
                    u"additional audio formats that'll play on different "
                    u"browsers and phones.",
        required=False,
        value_type=schema.Choice(
            vocabulary='plone.app.vocabularies.Catalog'
        )
    )

    @invariant
    def validate_audio_files(data):
        utils = getMultiAdapter((getSite(), getRequest()),
                                name="castle-utils")
        if data.audio_files:
            for audio in data.audio_files:
                obj = utils.get_object(audio)
                if obj.portal_type != 'Audio':
                    raise Invalid('Must provide only audio files')
        else:
            raise Invalid('Must provide audio file(s)')

    width = schema.TextLine(
        title=u"Width",
        default=u'100%',
        required=False
    )

    form.widget('display_fields', CheckBoxFieldWidget)
    display_fields = schema.Tuple(
        title=u'Display fields',
        description=u'Fields that should show from the content',
        default=(),
        value_type=schema.Choice(
            vocabulary=SimpleVocabulary([
                SimpleTerm('title', 'title', u'Title'),
                SimpleTerm('description', 'description', u'Overview/Summary'),
                SimpleTerm('date', 'date', u'Date'),
                SimpleTerm('transcript', 'transcript', u'Transcript'),
            ])
        )
    )



from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManager import setSecurityPolicy
from Acquisition import aq_parent
from Acquisition import ImplicitAcquisitionWrapper
from BTrees.OOBTree import OOBTree
from DateTime import DateTime
from datetime import datetime
from lxml.html import fromstring
from OFS.interfaces import IFolder
from Persistence.mapping import PersistentMapping as PM1  # noqa
from persistent.dict import PersistentDict
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping as PM2  # noqa
from plone.app.blob.field import BlobWrapper
from plone.app.blob.utils import openBlob
from Products.Archetypes import Field
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.tests.base.security import OmnipotentUser
from Products.CMFCore.tests.base.security import PermissiveSecurityPolicy
from Products.ZCatalog.Lazy import LazyCat
from StringIO import StringIO
from Testing.makerequest import makerequest
from ZODB.blob import Blob
from zope.component.hooks import setSite
from zope.dottedname.resolve import resolve
from ZPublisher.HTTPRequest import record

import argparse
import base64
import errno
import OFS
import os
import re


try:
    from Products.PressRoom.content.PressContact import PressContact
except:
    PressContact = False

try:
    from Products.CMFPlone import defaultpage
except:
    defaultpage = None

try:
    import json
except ImportError:
    import simplejson as json

try:
    from plone.app.contentlisting.contentlisting import ContentListing
except:
    ContentListing = object()


parser = argparse.ArgumentParser(
    description='...')
parser.add_argument('--overwrite', dest='overwrite', default=False)
parser.add_argument('--site-id', dest='site_id', default='Plone')
parser.add_argument('--dir', dest='dir', default='./export')
args, _ = parser.parse_known_args()


def spoofRequest(app):
    """
    Make REQUEST variable to be available on the Zope application server.

    This allows acquisition to work properly
    """
    _policy = PermissiveSecurityPolicy()
    _oldpolicy = setSecurityPolicy(_policy)  # noqa
    newSecurityManager(None, OmnipotentUser().__of__(app.acl_users))
    return makerequest(app)

app = spoofRequest(app)  # noqa

user = app.acl_users.getUser('admin')  # noqa
newSecurityManager(None, user.__of__(app.acl_users))  # noqa
site = app[args.site_id]
setSite(site)

_filedata_marker = 'filedata://'
_deferred_marker = 'deferred://'
_type_marker = 'type://'
_uid_marker = 'uid://'
_uid_separator = '||||'
_date_re = re.compile('^[0-9]{4}\-[0-9]{2}\-[0-9]{2}.*$')


export_folder = os.path.abspath(args.dir)
catalog = site.portal_catalog
workflow = site.portal_workflow
ptool = site.plone_utils
site_path = '/'.join(site.getPhysicalPath())


class BaseTypeSerializer(object):
    klass = None
    toklass = None

    @classmethod
    def getTypeName(cls):
        return "%s.%s" % (cls.klass.__module__, cls.klass.__name__)

    @classmethod
    def serialize(cls, obj):
        if hasattr(obj, 'aq_base'):
            obj = obj.aq_base
        data = cls._serialize(obj)
        results = {
            'type': cls.getTypeName(),
            'data': data
        }
        return _type_marker + dumps(results)

    @classmethod
    def _serialize(cls, obj):
        return cls.toklass(obj)
    @classmethod
    def deserialize(cls, data):
        return cls._deserialize(data)
    @classmethod
    def _deserialize(cls, data):
        return cls.klass(data)


class PM1Serializer(BaseTypeSerializer):
    klass = PM1
    toklass = dict


class PM2Serializer(PM1Serializer):
    klass = PM2


class PersistentDictSerializer(BaseTypeSerializer):
    klass = PersistentDict
    toklass = dict


class OOBTreeSerializer(BaseTypeSerializer):
    klass = OOBTree
    toklass = dict


class PersistentListSerializer(BaseTypeSerializer):
    klass = PersistentList
    toklass = list


class setSerializer(BaseTypeSerializer):
    klass = set
    toklass = list


class OFSFileSerializer(BaseTypeSerializer):
    klass = OFS.Image.File
    @classmethod
    def _serialize(cls, obj):
        try:
            data = str(obj.data)
        except:
            data = str(obj.data.data)
        return {
            'data': base64.b64encode(data),
            'id': obj.id(),
            'title': obj.title,
            'content_type': obj.content_type
        }
    @classmethod
    def _deserialize(cls, data):
        file = base64.b64decode(data['data'])
        id = data['id']
        title = data['title']
        ct = data['content_type']
        return cls.klass(id, title, file, ct)


class BlobSerializer(BaseTypeSerializer):
    klass = Blob
    @classmethod
    def _serialize(cls, obj):
        blobfi = openBlob(obj)
        data = blobfi.read()
        blobfi.close()
        return {'data': base64.b64encode(data)}
    @classmethod
    def _deserialize(cls, data):
        blob = Blob()
        bfile = blob.open('w')
        data = base64.b64decode(data['data'])
        bfile.write(data)
        bfile.close()
        return blob


class OFSImageSerializer(OFSFileSerializer):
    klass = OFS.Image.Image


class DateTimeSerializer(BaseTypeSerializer):
    klass = DateTime
    @classmethod
    def getTypeName(cls):
        return 'DateTime.DateTime'
    @classmethod
    def _serialize(cls, obj):
        return obj.ISO8601()
    @classmethod
    def _deserialize(cls, data):
        return DateTime(data)


class datetimeSerializer(BaseTypeSerializer):
    klass = datetime
    @classmethod
    def _serialize(cls, obj):
        return obj.isoformat()
    @classmethod
    def _deserialize(cls, data):
        return datetime.strptime(data, '%Y-%m-%dT%H:%M:%S.%f')


class recordSerializer(BaseTypeSerializer):
    klass = record
    toklass = dict
    @classmethod
    def _deserialize(cls, data):
        rec = record()
        for key, value in data.items():
            setattr(rec, key, value)
        return rec


class ContentListingSerializer(BaseTypeSerializer):
    klass = ContentListing
    toklass = list
    @classmethod
    def _serialize(cls, obj):
        return list(obj._basesequence)
    @classmethod
    def _deserialize(cls, data):
        return LazyCat(data)


class BlobWrapperSerializer(BaseTypeSerializer):
    klass = BlobWrapper
    @classmethod
    def _serialize(cls, obj):
        blob = obj.getBlob()
        blobfi = openBlob(blob)
        data = blobfi.read()
        blobfi.close()
        return {
            'data': base64.b64encode(data),
            'filename': obj.getFilename()}
    @classmethod
    def _deserialize(cls, data):
        io = StringIO(data['data'])
        io.filename = data['filename']
        return io

_serializers = {
    PM1: PM1Serializer,
    PM2: PM2Serializer,
    PersistentDict: PersistentDictSerializer,
    OOBTree: OOBTreeSerializer,
    PersistentList: PersistentListSerializer,
    set: setSerializer,
    OFS.Image.Image: OFSImageSerializer,
    Field.Image: OFSImageSerializer,
    OFS.Image.File: OFSFileSerializer,
    DateTime: DateTimeSerializer,
    datetime: datetimeSerializer,
    record: recordSerializer,
    Blob: BlobSerializer,
    ContentListing: ContentListingSerializer,
    BlobWrapper: BlobWrapperSerializer,
}


if PressContact:
    class ContentObjectSerializer(BaseTypeSerializer):
        klass = PressContact
        @classmethod
        def _serialize(cls, obj):
            return _uid_marker + obj.UID()
        @classmethod
        def _deserialize(cls, data):
            return None
    _serializers[PressContact] = ContentObjectSerializer

class Deferred:
    pass


def customhandler(obj):
    if hasattr(obj, 'aq_base'):
        obj = obj.aq_base
    _type = type(obj)
    if _type.__name__ == 'instance':
        _type = obj.__class__
    if _type in _serializers:
        serializer = _serializers[_type]
        return serializer.serialize(obj)
    else:
        return None
    return obj


def decodeUid(v):
    v = v[len(_uid_marker):]
    return v.split(_uid_separator)


def custom_decoder(d):
    if isinstance(d, list):
        pairs = enumerate(d)
    elif isinstance(d, dict):
        pairs = d.items()
    result = []
    for k, v in pairs:
        if isinstance(v, basestring):
            if v.startswith(_filedata_marker):
                if v == _filedata_marker + _deferred_marker:
                    v = Deferred
                else:
                    filedata = v[len(_filedata_marker):]
                    if filedata:
                        v = StringIO(base64.b64decode(filedata))
                    else:
                        v = ''
            elif v.startswith(_type_marker):
                v = v[len(_type_marker):]
                results = loads(v)
                _type = resolve(results['type'])
                serializer = _serializers[_type]
                v = serializer.deserialize(results['data'])
        elif isinstance(v, (dict, list)):
            v = custom_decoder(v)
        result.append((k, v))
    if isinstance(d, list):
        return [x[1] for x in result]
    elif isinstance(d, dict):
        return dict(result)


def loads(data):
    return json.loads(data, object_hook=custom_decoder)


def dumps(data):
    return json.dumps(data, default=customhandler)


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST:
            pass
        else:
            raise


def createPath(path):
    path = path[len(site_path):]
    xpath = os.path.join(export_folder, path.lstrip('/'))
    xpathsplit = xpath.split('/')
    xfolderpath = '/'.join(xpathsplit[:-1])
    mkdir_p(xfolderpath)
    return xpath


def getReferencedImages(data):
    images = []
    for key, value in data.items():
        if not isinstance(value, basestring):
            continue
        try:
            dom = fromstring(value)
        except:
            continue
        for el in dom.cssselect('img'):
            src = el.attrib.get('src', '')
            if 'resolveuid' not in src:
                continue
            uid = src.split('resolveuid/')[-1].split(
                '/@@images')[0].split('/image_')[0]
            brains = catalog(UID=uid)
            if len(brains) > 0:
                images.append(brains[0].getObject())
    return images


def exportObj(obj):
    data = {}

    im_width = 0
    image_to_lead = None
    for field in obj.Schema().fields():
        try:
            fdata = field.getRaw(obj)
        except ValueError:
            continue
        if type(fdata) == ImplicitAcquisitionWrapper:
            fdata = fdata.aq_base
        data[field.__name__] = fdata
        if field.__name__ in ('image', 'file'):
            data[field.__name__ + '_filename'] = field.getFilename(obj)
            data[field.__name__ + '_contentType'] = field.getContentType(obj)
        if field.__name__ == 'image':
            im = field.get(obj)
            if im:
                im_width = im.width

    images = getReferencedImages(data)

    # try to find image to associate with it if there is none
    if im_width < 200:
        if len(images) > 0 and images[0].portal_type == 'Image':
            image = images[0]
            if len(image.getBackReferences(relationship='isReferencing')) < 2:
                im = image.getImage()
                data.update({
                    'image': image.Schema().getField('image').getRaw(image),
                    'image_filename': image.getFilename(),
                    'image_contentType': image.getContentType(),
                    'imageCaption': image.Description()
                })
                image_to_lead = image.UID()
                images = images[1:]

    if image_to_lead and 'text' in data:
        data['text'] = data['text'].replace(image_to_lead, obj.UID())

    is_dp = False
    if not IFolder.providedBy(obj):
        if defaultpage:
            container = aq_parent(obj)
            if container:
                is_dp = defaultpage.is_default_page(container, obj)
        else:
            is_dp = ptool.isDefaultPage(obj)

    has_sibling_pages = True
    if is_dp:
        container = aq_parent(obj)
        res = catalog(path={'query': '/'.join(container.getPhysicalPath())})
        if len([b for b in res if b.UID != image_to_lead]) > 1:
            has_sibling_pages = False

    try:
        state = workflow.getInfoFor(ob=obj, name='review_state')
    except:
        state = None
    alldata = {
        'portal_type': obj.portal_type,
        'layout': obj.getLayout(),
        'data': data,
        'uid': obj.UID(),
        'is_default_page': is_dp,
        'state': state,
        'has_sibling_pages': has_sibling_pages
    }
    yield obj, alldata
    for ref in images:
        if ref.UID() == obj.UID():
            continue
        if ref.portal_type != 'Image':
            continue
        if image_to_lead == ref.UID():
            continue
        for o, d in exportObj(ref):
            if o.UID() == ref.UID() or o.UID() == obj.UID():
                continue
            yield o, d


def writeExport(obj, data):
    objpath = '/'.join(obj.getPhysicalPath())
    if IFolder.providedBy(obj):
        objpath = os.path.join(objpath, '__folder__')
    else:
        fobj = aq_parent(obj)
        while not ISiteRoot.providedBy(fobj):
            if not os.path.exists(os.path.join('/'.join(fobj.getPhysicalPath()), '__folder__')):
                for eobj, edata in exportObj(fobj):
                    writeExport(eobj, edata)
            fobj = aq_parent(fobj)
    path = createPath(objpath)
    try:
        fi = open(path, 'w')
        fi.write(dumps(data))
        fi.close()
    except UnicodeDecodeError:
        print('Error exporting {}'.format(objpath))


def runExport(brains):
    size = len(brains)
    for idx, brain in enumerate(brains):
        path = brain.getPath()
        print 'processing, ', path, ' ', str(idx + 1) + '/' + str(size)
        obj = brain.getObject()
        for obj, data in exportObj(obj):
            writeExport(obj, data)


runExport(catalog())
